/*
 * can.c
 *
 * Created: 03.02.2016 18.05.28
 *  Author: Simen
 */ 

#include <avr/interrupt.h>
#include <stdlib.h>
#include <stdbool.h>

#include "can.h"
#include "can_drv.h"

uint8_t can_setup_receive_buffer(can_msg_buffer *buff, uint8_t mob);

#define TX_MOB 0

void *(*can_callback_function[CAN_MAX_RX_BUFFERS])(uint8_t status);


void can_init(	enum can_cpu_clock clock, 
				enum can_baud_rate baudrate,
				can_msg_buffer *rx_buffers[CAN_MAX_RX_BUFFERS],
				uint8_t num_buffers)
{
	Can_reset();
	
	if(baudrate == AUTO_BAUD){
		can_auto_baudrate(0);
	}else{
		can_setBaudrate(clock, baudrate);
	}
	
	//Clean all mailboxes
	can_clear_all_mob();
	
	
	if(num_buffers > (NB_MOB - 1)){
		return;
	}
	for(uint8_t i = 0; i < num_buffers; i++){
		
		can_setup_receive_buffer(rx_buffers[i], i+1);
	}
	
	CANIE2 |= (1 << 0);
	CANGIE |= (1<<ENIT)|(1 << ENTX);
	
	Can_enable();
}


uint8_t can_send_message(can_msg_t *msg){
	
	Can_set_mob(TX_MOB);
	
	uint8_t tx_state = can_get_mob_status();
	
	if(tx_state != MOB_TX_COMPLETED && tx_state != MOB_DISABLE){
		return CAN_STATUS_NOT_COMPLETED;
	}
	
	Can_set_mob(TX_MOB);
	Can_clear_mob();
	
	if (msg->extended_ID){ 
		Can_set_ext_id(msg->id.ext);
	}
	else{
		Can_set_std_id(msg->id.std);
	}
	for (uint8_t i = 0; i < msg->dlc; i++){
		CANMSG = msg->pt_data->u8[i];
	}
	
	Can_clear_rtr();
	
	Can_set_dlc(msg->dlc);
	Can_config_tx();
	
	return CAN_STATUS_COMPLETED;
}

uint8_t can_setup_receive_buffer(can_msg_buffer *buffer, uint8_t mob){
	
	if(mob == TX_MOB){
		return CAN_STATUS_NOT_COMPLETED;
	}
	
	Can_set_mob(mob);
	Can_clear_mob();
	
	if (buffer->ide){
		Can_set_ext_id(buffer->id.ext);
		Can_set_ext_msk(buffer->mask.ext);}
	else{
		Can_set_std_id(buffer->id.std);
		Can_set_std_msk(buffer->mask.std);
	}
	
	if(buffer->callback_function != NULL){
		CANIE2 |= (1 << mob);
		
		//enable receive interrupt
		CANGIE |= (1<< ENRX);
		
		can_callback_function[mob-1] = buffer->callback_function;
	}
	
	Can_clear_rtr();
	Can_config_rx();
	
	buffer->handle = mob;
	
	return CAN_STATUS_COMPLETED;
}

uint8_t can_get_rx_buffer_state(can_msg_buffer *buffer){
	
	Can_set_mob(buffer->handle);
	
	uint8_t status = can_get_mob_status();
	switch(status){
		case MOB_RX_COMPLETED:
		case MOB_RX_COMPLETED_DLCW:
			return CAN_STATUS_COMPLETED;
		case MOB_NOT_COMPLETED:
		case MOB_TX_COMPLETED:
			return CAN_STATUS_NOT_COMPLETED;
		default:
			return CAN_STATUS_ERROR;
	}
	return CAN_STATUS_ERROR;
}

void can_reinit_rx_buffer(can_msg_buffer *buffer){
	Can_set_mob(buffer->handle);
	
	Can_mob_abort();
	Can_clear_status_mob();
	Can_clear_mob();
	
	can_setup_receive_buffer(buffer, buffer->handle);
}

uint8_t can_get_rx_buffer_data(can_msg_buffer *buffer, can_msg_t *out){
	
	uint8_t status = can_get_rx_buffer_state(buffer);
	
	if(status != CAN_STATUS_COMPLETED){
		return status;
	}
	
	if(out != NULL){
		
		Can_set_mob(buffer->handle);
	
		out->dlc = Can_get_dlc();
		can_get_data(out->pt_data);
		if (Can_get_ide()) // if extended frame
		{
			out->extended_ID = true; // extended frame
			Can_get_ext_id(out->id.ext);
		}
		else // else standard frame
		{
			out->extended_ID = false;
			Can_get_std_id(out->id.std);
		}
	}
	
	can_reinit_rx_buffer(buffer);	
	
	return CAN_STATUS_COMPLETED;
}

ISR(CAN_INT_vect){
	//TX interrupt? Stop transmission
	Can_set_mob(TX_MOB);
	if(can_get_mob_status() == MOB_TX_COMPLETED){
		//Disable TX mob to stop transmission
		Can_mob_abort();
		Can_clear_status_mob();
		Can_clear_mob();
		/*Can_clear_status_mob();
		DISABLE_MOB;*/
	}
	for(uint8_t i = 0; i < CAN_MAX_RX_BUFFERS; i++){
		Can_set_mob(i + 1);
		uint8_t status = can_get_mob_status();
		if(status != MOB_NOT_COMPLETED && status != MOB_DISABLE && can_callback_function[i] != NULL){		
			can_callback_function[i](status);
		}
	}
}