/*
 * can.h
 *
 * Created: 03.02.2016 18.05.43
 *  Author: Simen
 */ 


#ifndef CAN_H_
#define CAN_H_
#include <stdbool.h>

#define CAN_MAX_RX_BUFFERS			5
#define CAN_STATUS_COMPLETED		0x00
#define CAN_STATUS_NOT_COMPLETED	0x01
#define CAN_STATUS_ERROR			0x02

enum can_cpu_clock{clkph_8MHZ,clkph_12MHZ, clkph_16MHZ };
enum can_baud_rate{CAN_BPS_1000k, CAN_BPS_500K, CAN_BPS_250K, CAN_BPS_200K, CAN_BPS_125K, CAN_BPS_100K, AUTO_BAUD};

typedef union{
	uint32_t ext;
	uint16_t std;
} can_id_t;

typedef union{
	uint8_t		u8[8];
	int8_t		i8[8];
	uint16_t	u16[4];
	int16_t		i16[4];
	uint32_t	u32[2];
	int32_t		i32[2];
	uint64_t	u64;
	int64_t		i64;
} can_data_t;

typedef  struct{
	can_id_t		id;
	uint8_t         dlc;
	can_data_t*     pt_data;
	bool			extended_ID;
	bool			rtr;
} can_msg_t;

//Data necessary to setup a message transmission.
//ID and mask define the reception filter
//IDE and RTR enable extended ID length and Remote transmission frames

typedef struct{
	uint8_t     handle;
	can_id_t	id;
	can_id_t	mask;
	bool		ide;
	bool		rtr;
	void *		(*callback_function)(uint8_t status);
} can_msg_buffer;

//Automatically enables message reception with settings defined in rx_buffers

void can_init(	enum can_cpu_clock clock,
				enum can_baud_rate baudrate,
				can_msg_buffer *rx_buffers[CAN_MAX_RX_BUFFERS],
				uint8_t num_buffers);

uint8_t can_send_message(can_msg_t *msg);
uint8_t can_get_rx_buffer_state(can_msg_buffer *buff);
uint8_t can_get_rx_buffer_data(can_msg_buffer *buffer, can_msg_t *out);

#endif /* CAN_H_ */