/*
 * sensor_temperature.c
 *
 * Created: 06/03/2016 12:56:46
 *  Author: anders
 */
#include <avr/io.h>
#include <stdbool.h>
#include <stdlib.h>
#include "sensor_conversion.h"

//translate ADC value to 0-5V signal with 1/100 V precision. (0-500)
static int16_t sensor_adc_to_mVin(int16_t adc_value){
	return (((adc_value*ADC_REF_VOLTAGE)*5)/1023)*20;	// int16_t formula
}
/*
* return (((adc_value*ADC_REF_VOLTAGE)*10)/1023)*10; // uint16_t	 formula
*/

// Returns degrees @ 1/100th precision. (1500-15000 "centidegrees" C)
int16_t sensor_adc_to_temp(int16_t adc_value, bool motorTemp){
	int16_t dVin = sensor_adc_to_mVin(adc_value);
	if(motorTemp){
		return 41703 - 190*dVin;
	}else{
		return 46*dVin - 452;
	}
}

/*
* Motor temp: 
* return 48379 - 227*dVin; // regression 10-300
* NTCLE413-428
* return -1612 + 42*dVin;
*/


int16_t sensor_adc_to_vdc(int16_t adc_value){
	int16_t dVin = sensor_adc_to_mVin(adc_value);
	return (dVin*((R1+R2)/R2))/100;
}

void sensor_hysteresis(int16_t onVal, int16_t offVal, TempData *temp){
	if(!temp->hysteresis){ //hysteresis has not activated
		if(temp->temp >= onVal){temp->hysteresis = true;}
	}else{ // hysteresis is active
		if(temp->temp < offVal){temp->hysteresis = false;}
	}
}