	/*
 * revolve_can_definitions.h
 *
 * Created: 20.01.2013 18:47:51
 *  Author: Bruker
 */ 


#ifndef REVOLVE_CAN_DEFINITIONS_H_
#define REVOLVE_CAN_DEFINITIONS_H_

//ID
//Module ID
//Bytemapping

//------------------------------------------------------------------------------
// EXAMPLE: function = command, group = dash, module ID = 0
/* .id = CANR_FCN_CMD_ID | CANR_GRP_DASH_ID | CANR_MODULE_ID0 */
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
/*	-------	ARBITRATION ID DEFINITIONS ---------------------------------------*/
//------------------------------------------------------------------------------

//- ID[10..8]
#define CANR_FCN_INVSET_ID			0x100		// Most significant
#define CANR_FCN_INVDATA_ID			0x200		// Priority data
#define CANR_FCN_PRI_ID				0x300		// Module_id control
#define CANR_FCN_DATA_ID			0x400		// Data
#define CANR_FCN_BMSDATA_ID			0x500

#define CANR_FCN_MASK				0x700

//- ID[7..4]
#define CANR_GRP_ADC_DAMPER_ID			0x00		 
#define CANR_GRP_ADC_DRIVERINPUT_ID		0x10		 
#define CANR_GRP_ADC_TEMP_ID			0x20		 
#define CANR_GRP_ADC_BSPD_ID			0x30		 
#define CANR_GRP_BMS_ID					0x40		
#define CANR_GRP_ECU_ID					0x50		
#define CANR_GRP_GLVBMS_ID				0x60		
#define CANR_GRP_DASH_ID				0x70		
#define CANR_GRP_BSPD_ID	 			0x80		
#define CANR_GRP_IMD_ID					0x90		
#define CANR_GRP_TELEMETRY_ID			0xA0		
#define CANR_GRP_FAN_CTRL_ID			0xB0
#define CANR_GRP_ALIVE_ID				0xC0
#define CANR_GRP_SFII_ID				0xD0		

#define CANR_GRP_MASK					0x0F0

// The inverter address is calculated with 282, 284, or 183  + node adress
// 183 + node address is an address field used by the ECU to send setpoints to the Inverters
// But I dont see solution with this setup to allocate that room only to the ECU
#define CANR_GRP_INVERTER_AMK_ID		0x80
#define CANR_GRP_INVERTER_R16_ID		0x90							
// Optical sensor has programmable address, so it can be added when we get it.

//- ID[3..0]
#define CANR_MODULE_ID0_ID			0x00
#define CANR_MODULE_ID1_ID			0x01
#define CANR_MODULE_ID2_ID			0x02
#define CANR_MODULE_ID3_ID			0x03
#define CANR_MODULE_ID4_ID			0x04
#define CANR_MODULE_ID5_ID			0x05
#define CANR_MODULE_ID6_ID			0x06
#define CANR_MODULE_ID7_ID			0x07
#define CANR_MODULE_ID8_ID			0x08
#define CANR_MODULE_ID9_ID			0x09
#define CANR_MODULE_ID10_ID			0x0A
#define CANR_MODULE_ID11_ID			0x0B
#define CANR_MODULE_ID12_ID			0x0C
#define CANR_MODULE_ID13_ID			0x0D
#define CANR_MODULE_ID14_ID			0x0E
#define CANR_MODULE_ID15_ID			0x0F

#define CANR_MODULE_ID_MASK		0x00F;



//------------------------------------------------------------------------------
/*	-------	ALIVE	 ---------------------------------------------------------*/
//------------------------------------------------------------------------------

// ---------------------------------------------------------------------//
// Functionality description:											//
// Each module sends out a message with the following properties:		//
// ID = CANR_FCN_DATA_ID | CANR_GRP_DASH_ID	| CANR_MODULE_ID7_ID		//
// DATA[0] = MODULE ID													//
// Frequency = 1 Hz														//
// ---------------------------------------------------------------------//

//-ALIVE ID
#define CANR_ALIVE_MSG_ID			(CANR_FCN_DATA_ID	|CANR_GRP_ALIVE_ID | CANR_MODULE_ID0_ID)
#define CANR_ALIVE_MSG_DLC			1



//Module Alive IDs


#define ALIVE_ADC_BSPD   			0x01
#define ALIVE_ADC_TEMPFRONT			0x02
#define ALIVE_ADC_TEMPREAR			0x03
#define ALIVE_ADC_DAMPERFRONT		0x04
#define ALIVE_ADC_DAMPERREAR	 	0x05
#define ALIVE_ADC_PEDALS			0x06
#define ALIVE_ECU					0x07
#define ALIVE_TELEMETRY				0x08
#define ALIVE_DASH					0x09
#define ALIVE_FAN					0x0A
#define ALIVE_BMS					0x0B
#define ALIVE_GLVBMS				0x0C
#define ALIVE_IMD					0x0D
#define ALIVE_INVERTER_0			0x10
#define ALIVE_INVERTER_1			0x11
#define ALIVE_INVERTER_2			0x12
#define ALIVE_INVERTER_3			0x13

//-------------------------------------------------------------------------------
/*	------- Module ID overrides	 ----------------------------------------------*/
/*   If you want to make your own macro to replace module_id_x, make it here   */
//-------------------------------------------------------------------------------

//-------------------------------------------------------------------------------
//                          Parameter setting CAN IDS
//-------------------------------------------------------------------------------

#define PARAMETER_SET_ID					(CANR_FCN_PRI_ID | CANR_GRP_TELEMETRY_ID | CANR_MODULE_ID0_ID)
#define PARAMETER_SET_MODULE_ID_8pos		0
#define PARAMETER_SET_PARAM_ID_8pos			1
#define PARAMETER_SET_PARAM_val_32pos		1		

//-------------------------------------------------------------------------------
//                          Status bit setting CAN IDS
//-------------------------------------------------------------------------------

#define STATUS_BIT_SET_ID					(CANR_FCN_PRI_ID | CANR_GRP_TELEMETRY_ID | CANR_MODULE_ID1_ID)
#define STATUS_BIT_SET_MODULE_ID_8pos		0
#define STATUS_BIT_SET_OPERATION_ID_8pos	1
#define STATUS_BIT_SET_PARAM_mask_32pos		1

#define STATUS_BIT_SET_OPERATION_INVALID	0
#define STATUS_BIT_SET_OPERATION_SET		1
#define STATUS_BIT_SET_OPERATION_CLEAR		2
#define STATUS_BIT_SET_OPERATION_FLIP		4		

//-------------------------------------------------------------------------------
//                         SFII CAN IDS
//-------------------------------------------------------------------------------
// UINT16[0] : V_x, C = 0.01 ||| INT16[1] : V_y, C = 0.01 ||| INT16[2] SIDESLIPANGLE, C = 0.01
#define SFII_DATA_FRAME_2_ID				(CANR_FCN_PRI_ID  | CANR_GRP_SFII_ID | CANR_MODULE_ID0_ID) // Contains Vx,Vy,SA
// NO NEED TO PARSE DATA_FRAME_1
#define SFII_DATA_FRAME_1_ID				(CANR_FCN_DATA_ID | CANR_GRP_SFII_ID | CANR_MODULE_ID0_ID)
// NO NEED TO PARSE STATUS
#define SFII_STATUS_ID						(CANR_FCN_DATA_ID | CANR_GRP_SFII_ID | CANR_MODULE_ID1_ID)
// NO NEED TO PARSE_CONTROL_FRAME
#define SFII_CONTROL_FRAME_ID				(CANR_FCN_DATA_ID | CANR_GRP_SFII_ID | CANR_MODULE_ID2_ID)

//-------------------------------------------------------------------------------
//                          ECU CAN IDS
//-------------------------------------------------------------------------------
// JACOB ALREADY HAS INFORMATION TO PARSE SETPOINTS TO AMK , AND DATA FROM AMK
#define ECU_SET_POINT_AMK_FL_ID				(CANR_FCN_INVSET_ID | CANR_GRP_INVERTER_AMK_ID | CANR_MODULE_ID3_ID)
#define ECU_SET_POINT_AMK_FR_ID				(CANR_FCN_INVSET_ID | CANR_GRP_INVERTER_AMK_ID | CANR_MODULE_ID4_ID)
#define ECU_SET_POINT_AMK_RL_ID				(CANR_FCN_INVSET_ID | CANR_GRP_INVERTER_AMK_ID | CANR_MODULE_ID5_ID)
#define ECU_SET_POINT_AMK_RR_ID				(CANR_FCN_INVSET_ID | CANR_GRP_INVERTER_AMK_ID | CANR_MODULE_ID6_ID)

#define ECU_PLAY_RTDS_ID					(CANR_FCN_PRI_ID | CANR_GRP_ECU_ID | CANR_MODULE_ID0_ID)
#define ECU_DRIVE_ENABLE_ID					(CANR_FCN_PRI_ID | CANR_GRP_ECU_ID | CANR_MODULE_ID1_ID)
#define ECU_DRIVE_DISABLE_ID				(CANR_FCN_PRI_ID | CANR_GRP_ECU_ID | CANR_MODULE_ID2_ID)
// UINT32[0] : 32 bit STATUS WORD
#define ECU_STATUS_MESSAGE_ID				(CANR_FCN_DATA_ID | CANR_GRP_ECU_ID | CANR_MODULE_ID0_ID)
// INT16[0] : FZ_DAMPER_FL , C = 10^-3 ||| INT16[1] : FZ_DAMPER_FR , C = 10^-3
// INT16[2] : FZ_DAMPER_RL , C = 10^-3 ||| INT16[3] : FZ_DAMPER_RR ,  C = 10^-3
#define ECU_SLIP_RATIO_ID					(CANR_FCN_DATA_ID | CANR_GRP_ECU_ID | CANR_MODULE_ID1_ID)
// UINT16[0] : FX_DIV_FZ_FL , C = 0.1 ||| UINT16[1] : FX_DIV_FZ_FR , C = 0.1
// UINT16[2] : FX_DIV_FZ_RL , C = 0.1 ||| UINT16[3] : FX_DIV_FZ_RR , C = 0.1
#define ECU_FX_DIV_FZ_ID					(CANR_FCN_DATA_ID | CANR_GRP_ECU_ID | CANR_MODULE_ID2_ID)
// UINT16[0] : FZ_DAMPER_FL , C = 0.1 ||| UINT16[1] : FZ_DAMPER_FR , C = 0.1
// UINT16[2] : FZ_DAMPER_RL , C = 0.1 ||| UINT16[3] : FZ_DAMPER_RR , C = 0.1
#define ECU_FZ_DAMPER_EST_ID				(CANR_FCN_DATA_ID | CANR_GRP_ECU_ID | CANR_MODULE_ID3_ID)
// NO NEED TO PARSE THIS ATM
#define ECU_FZ_LOAD_TRANSFER_ID				(CANR_FCN_DATA_ID | CANR_GRP_ECU_ID | CANR_MODULE_ID4_ID)
// INT16[0] : MZ_REF , C = 0.1 ||| INT16[1] : YAW_RATE_REF , C = 0.01
#define ECU_CONTROL_SYSTEM_VALUES_ID		(CANR_FCN_DATA_ID | CANR_GRP_ECU_ID | CANR_MODULE_ID5_ID)

// Data from INS that must be passed onto the CAN bus
// INT32[0] : LONGITUDE, C = 10^-6 ||| INT32[1] : LATITUDE, C = 10^-6
#define ECU_GPS_LONGITUDE_LATITUDE_ID		(CANR_FCN_DATA_ID | CANR_GRP_ECU_ID | CANR_MODULE_ID6_ID)
// NOT SENDING THIS ATM. NEEDED ? 
#define ECU_GPS_ALTITUDE_ID					(CANR_FCN_DATA_ID | CANR_GRP_ECU_ID | CANR_MODULE_ID7_ID)
// INT16[0] : YAW RATE [rad/s], C = 10^-3 ||| INT16[1] : YAW_ACCELERATION [rad/s^2] , C = 10^-2
#define ECU_INS_YAW_RATE_ACC_ID				(CANR_FCN_DATA_ID | CANR_GRP_ECU_ID | CANR_MODULE_ID8_ID)
// INT16[0] : A_x , C = 10^-2 ||| INT16[1] : A_y , C = 10^-2 ||| INT16[2] : INS_V_x , C = 10^-2 ||| INT16[3] : INS_V_y , C = 10^-2
#define ECU_INS_AX_AY_VX_VY_ID				(CANR_FCN_DATA_ID | CANR_GRP_ECU_ID | CANR_MODULE_ID9_ID)
// UINT16[0] : STATUS_ATTITUDE ||| UINT8[2] : STATUS_INS
#define ECU_INS_STATUSES_ID					(CANR_FCN_DATA_ID | CANR_GRP_ECU_ID | CANR_MODULE_ID10_ID)
// UINT8[0] : GPS_FIX  ||| UINT8[1] : NUMBER OF TRACKED SATELITES 
#define ECU_GPS_FIX_AND_NRSATS_ID			(CANR_FCN_DATA_ID | CANR_GRP_ECU_ID | CANR_MODULE_ID11_ID)

// ECU Parameters
#define PARAM_ECU_COMMAND_USE_DEFAULT_PARAMETERS								200
#define PARAM_ECU_COMMAND_ECU_SEND_CURRENT_PARAMETERS_ON_CAN					201
#define PARAM_ECU_TV_METHOD														0
#define PARAM_ECU_SELECT_FZ_METHOD												1
#define PARAM_ECU_SELECT_VX														2
#define PARAM_ECU_SELECT_VY														3
#define PARAM_ECU_FRICTION_COEFFICIENT											4
#define PARAM_ECU_DRIVER_MASS													5
#define PARAM_ECU_EFFECTIVE_WHEEL_RADIUS										120

#define PARAM_ECU_YAW_RATE_CONTROL_KU											6							
#define PARAM_ECU_YAW_RATE_CONTROL_GAIN_PROPORTIONAL_START						7
#define PARAM_ECU_YAW_RATE_CONTROL_GAIN_PROPORTIONAL_END						8
#define PARAM_ECU_YAW_RATE_CONTROL_GAIN_PROPORTIONAL_SCALING					9
#define PARAM_ECU_YAW_RATE_CONTROL_GAIN_INTEGRAL_START							10
#define PARAM_ECU_YAW_RATE_CONTROL_GAIN_INTEGRAL_END							11
#define PARAM_ECU_YAW_RATE_CONTROL_GAIN_INTEGRAL_SCALING						12
#define PARAM_ECU_YAW_RATE_CONTROL_GAIN_SATURATION								13
#define PARAM_ECU_YAW_RATE_CONTROL_GAIN_DERIVATIVE_START						14
#define PARAM_ECU_YAW_RATE_CONTROL_GAIN_DERIVATIVE_END							15
#define PARAM_ECU_YAW_RATE_CONTROL_GAIN_DERIVATIVE_SCALING						16
#define PARAM_ECU_YAW_RATE_CONTROL_LOOKUP_SPEED_END_KMH							17
#define PARAM_ECU_YAW_RATE_CONTROL_REFERENCE_WEIGHT								18
#define PARAM_ECU_YAW_RATE_CONTROL_MEASUREMENT_WEIGHT							19	
#define PARAM_ECU_YAW_RATE_CONTROL_ENABLE_R_REF_LIMIT							20				
#define PARAM_ECU_YAW_RATE_CONTROL_MU_SCALING_R_REF_LIMIT						21
#define PARAM_ECU_YAW_RATE_CONTROL_ENABLE										22

#define PARAM_ECU_TRACTION_CONTROL_SLIP_RATIO_REFERENCE							23
#define PARAM_ECU_TRACTION_CONTROL_GAIN_KP_START								24
#define PARAM_ECU_TRACTION_CONTROL_GAIN_KP_END									25
#define PARAM_ECU_TRACTION_CONTROL_GAIN_KP_SCALING								26
#define PARAM_ECU_TRACTION_CONTROL_GAIN_KI_START								27
#define PARAM_ECU_TRACTION_CONTROL_GAIN_KI_END									28
#define PARAM_ECU_TRACTION_CONTROL_GAIN_KI_SCALING								29
#define PARAM_ECU_TRACTION_CONTROL_GAIN_KP_BRAKING								30
#define PARAM_ECU_TRACTION_CONTROL_GAIN_KI_BRAKING								31
#define PARAM_ECU_TRACTION_CONTROL_LOOKUP_SPEED_END_KMH							32
#define PARAM_ECU_TRACTION_CONTROL_GAIN_ANTI_WIND_UP							33
#define PARAM_ECU_TRACTION_CONTROL_ENABLE										34
																				
#define PARAM_ECU_POWER_LIMIT_CONTROL_POSITIVE_POWER_LIMIT						35
#define PARAM_ECU_POWER_LIMIT_CONTROL_NEGATIVE_POWER_LIMIT						36
#define PARAM_ECU_POWER_LIMIT_CONTROL_GAIN_KP									37
#define PARAM_ECU_POWER_LIMIT_CONTROL_GAIN_KI									38
#define PARAM_ECU_POWER_LIMIT_CONTROL_GAIN_ANTI_WIND_UP							39
#define PARAM_ECU_POWER_LIMIT_CONTROL_ENABLE									40





//-------------------------------------------------------------------------------
//                          AMK INVERTER CAN IDS
//-------------------------------------------------------------------------------
//AMK_DATA_ACTUAL_VALUE_1 STRUCTURE:
#define AMK_DATA_ACTUAL_VALUE_1_FL_ID		(CANR_FCN_INVDATA_ID | CANR_GRP_INVERTER_AMK_ID | CANR_MODULE_ID2_ID)
#define AMK_DATA_ACTUAL_VALUE_2_FL_ID		(CANR_FCN_INVDATA_ID | CANR_GRP_INVERTER_AMK_ID | CANR_MODULE_ID4_ID)

#define AMK_DATA_ACTUAL_VALUE_1_FR_ID		(CANR_FCN_INVDATA_ID | CANR_GRP_INVERTER_AMK_ID | CANR_MODULE_ID3_ID)
#define AMK_DATA_ACTUAL_VALUE_2_FR_ID		(CANR_FCN_INVDATA_ID | CANR_GRP_INVERTER_AMK_ID | CANR_MODULE_ID5_ID)

#define AMK_DATA_ACTUAL_VALUE_1_RL_ID		(CANR_FCN_INVDATA_ID | CANR_GRP_INVERTER_AMK_ID | CANR_MODULE_ID4_ID)
#define AMK_DATA_ACTUAL_VALUE_2_RL_ID		(CANR_FCN_INVDATA_ID | CANR_GRP_INVERTER_AMK_ID | CANR_MODULE_ID6_ID)

#define AMK_DATA_ACTUAL_VALUE_1_RR_ID		(CANR_FCN_INVDATA_ID | CANR_GRP_INVERTER_AMK_ID | CANR_MODULE_ID5_ID)
#define AMK_DATA_ACTUAL_VALUE_2_RR_ID		(CANR_FCN_INVDATA_ID | CANR_GRP_INVERTER_AMK_ID | CANR_MODULE_ID7_ID)

//-------------------------------------------------------------------------------
//                          R16 INVERTER CAN IDS
//-------------------------------------------------------------------------------
#define R16_DATA_MECHANICAL_ID(n)			((n << 2) | (CANR_FCN_INVDATA_ID | CANR_GRP_INVERTER_R16_ID | CANR_MODULE_ID0_ID))
#define R16_DATA_TEMPERATURE_ID(n)			((n << 2) | (CANR_FCN_INVDATA_ID | CANR_GRP_INVERTER_R16_ID | CANR_MODULE_ID1_ID))
#define R16_DATA_ELECTRICAL_ID(n)			((n << 2) | (CANR_FCN_INVDATA_ID | CANR_GRP_INVERTER_R16_ID | CANR_MODULE_ID2_ID))
#define R16_STATUS_ID(n)					((n << 2) | (CANR_FCN_INVDATA_ID | CANR_GRP_INVERTER_R16_ID | CANR_MODULE_ID3_ID))
#define R16_PARAMETERS						(CANR_FCN_INVSET_ID  | CANR_GRP_INVERTER_R16_ID | CANR_MODULE_ID0_ID)
#define R16_RESERVED_0						(CANR_FCN_INVSET_ID  | CANR_GRP_INVERTER_R16_ID | CANR_MODULE_ID1_ID)
#define R16_RESERVED_1						(CANR_FCN_INVSET_ID  | CANR_GRP_INVERTER_R16_ID | CANR_MODULE_ID2_ID)
#define R16_RESERVED_2						(CANR_FCN_INVSET_ID  | CANR_GRP_INVERTER_R16_ID | CANR_MODULE_ID3_ID)

#define R16_TELEMETRY_TORQUE_REQUEST_ID		(CANR_FCN_PRI_ID	 | CANR_GRP_TELEMETRY_ID	| CANR_MODULE_ID2_ID)
//Data format
#define R16_MECHANICAL_RPM_16pos			0
#define R16_FOC_IQ_16pos					0
#define R16_FOC_ID_16pos					1
#define R16_DC_VOLTAGE_16pos				0
#define R16_DC_CURRENT_16pos				1
#define R16_STATUS_CURRENT_STATE_32pos		0
#define R16_STATUS_ACCUMULATED_STATE_32pos	1	

//R16 Parameters
#define PARAM_R16_K_p_Q						0
#define PARAM_R16_K_I_Q						1
#define PARAM_R16_K_a_Q						2
#define PARAM_R16_K_p_D						3
#define PARAM_R16_K_I_D						4
#define PARAM_R16_K_a_d						5
#define PARAM_R16_K_p_FW_up					6
#define PARAM_R16_K_p_FW_dn					7

//-------------------------------------------------------------------------------
//                          DASH CAN IDS
//-------------------------------------------------------------------------------
#define DASH_REQUEST_DRIVE_ENABLE_ID				(CANR_FCN_DATA_ID | CANR_GRP_DASH_ID | CANR_MODULE_ID0_ID)
#define DASH_REQUEST_DRIVE_DISABLE_ID				(CANR_FCN_DATA_ID | CANR_GRP_DASH_ID | CANR_MODULE_ID1_ID)
#define DASH_RTDS_FINISHED_ID						(CANR_FCN_DATA_ID | CANR_GRP_DASH_ID | CANR_MODULE_ID2_ID)
#define DASH_REQUEST_ACTIVATE_LAUNCH_CONTROL_ID		(CANR_FCN_DATA_ID | CANR_GRP_DASH_ID | CANR_MODULE_ID3_ID)
#define DASH_REQUEST_DEACTIVATE_LAUNCH_CONTROL_ID	(CANR_FCN_DATA_ID | CANR_GRP_DASH_ID | CANR_MODULE_ID4_ID)
#define DASH_KERS_REQUEST_ID						(CANR_FCN_DATA_ID | CANR_GRP_DASH_ID | CANR_MODULE_ID5_ID)
#define DASH_CURRENT_MENU_ID						(CANR_FCN_DATA_ID | CANR_GRP_DASH_ID | CANR_MODULE_ID6_ID)
#define DASH_FAN_CONTROL_SETTINGS_ID				(CANR_FCN_DATA_ID | CANR_GRP_DASH_ID | CANR_MODULE_ID7_ID)
#define DASH_TRACTION_CONTROL_REQUEST_ID			(CANR_FCN_DATA_ID | CANR_GRP_DASH_ID | CANR_MODULE_ID8_ID)
#define DASH_YAW_RATE_CONTROL_REQUEST_ID			(CANR_FCN_DATA_ID | CANR_GRP_DASH_ID | CANR_MODULE_ID9_ID)
#define DASH_TORQUE_ALLOCATION_REQUEST_ID			(CANR_FCN_DATA_ID | CANR_GRP_DASH_ID | CANR_MODULE_ID10_ID)
#define DASH_POWER_LIMIT_CONTROL_REQUEST_ID			(CANR_FCN_DATA_ID | CANR_GRP_DASH_ID | CANR_MODULE_ID11_ID)
//-------------------------------------------------------------------------------
//                          ADC CAN IDS
//-------------------------------------------------------------------------------
#define ADC_DAMPER_FL_ID							(CANR_FCN_DATA_ID | CANR_GRP_ADC_DAMPER_ID | CANR_MODULE_ID0_ID)
#define ADC_DAMPER_FR_ID							(CANR_FCN_DATA_ID | CANR_GRP_ADC_DAMPER_ID | CANR_MODULE_ID1_ID)
#define ADC_DAMPER_RL_ID							(CANR_FCN_DATA_ID | CANR_GRP_ADC_DAMPER_ID | CANR_MODULE_ID2_ID)
#define ADC_DAMPER_RR_ID							(CANR_FCN_DATA_ID | CANR_GRP_ADC_DAMPER_ID | CANR_MODULE_ID3_ID)

#define ADC_STEERING_ID								(CANR_FCN_DATA_ID | CANR_GRP_ADC_DRIVERINPUT_ID | CANR_MODULE_ID0_ID)
#define ADC_TORQUE_POSTION_LEFT_ID					(CANR_FCN_DATA_ID | CANR_GRP_ADC_DRIVERINPUT_ID | CANR_MODULE_ID1_ID)
#define ADC_TORQUE_POSTION_RIGHT_ID					(CANR_FCN_DATA_ID | CANR_GRP_ADC_DRIVERINPUT_ID | CANR_MODULE_ID2_ID)
#define ADC_KERS_POSITION_ID						(CANR_FCN_DATA_ID | CANR_GRP_ADC_DRIVERINPUT_ID | CANR_MODULE_ID3_ID)
#define ADC_BRAKE_PRESSURE_FRONT_AND_REAR_ID		(CANR_FCN_DATA_ID | CANR_GRP_ADC_DRIVERINPUT_ID | CANR_MODULE_ID4_ID)

#define ADC_GEAR_TEMPERATURE_FRONT_ID				(CANR_FCN_DATA_ID | CANR_GRP_ADC_TEMP_ID | CANR_MODULE_ID0_ID)
#define ADC_GEAR_TEMPERATURE_REAR_ID				(CANR_FCN_DATA_ID | CANR_GRP_ADC_TEMP_ID | CANR_MODULE_ID1_ID)
#define ADC_COOLANT_TEMPERATURE_LEFT_ID				(CANR_FCN_DATA_ID | CANR_GRP_ADC_TEMP_ID | CANR_MODULE_ID2_ID)
#define ADC_COOLANT_TEMPERATURE_RIGHT_ID			(CANR_FCN_DATA_ID | CANR_GRP_ADC_TEMP_ID | CANR_MODULE_ID3_ID)

//-------------------------------------------------------------------------------
//                          BSPD CAN IDS
//-------------------------------------------------------------------------------
#define BSPD_TRIGGER_ID								(CANR_FCN_DATA_ID | CANR_GRP_BSPD_ID	 | CANR_MODULE_ID0_ID)

//-------------------------------------------------------------------------------------

//-------------------------------------------------------------------------------
//                          BMS CAN IDS
//-------------------------------------------------------------------------------
#define CANR_GRP_BMS_TEMPERATURE_ID		0x00
#define CANR_GRP_BMS_VOLTAGE_ID				0x40

#define BMS_CELL_VOLTAGE(n)					(CANR_FCN_BMSDATA_ID | CANR_GRP_BMS_VOLTAGE_ID | (n))					// uint16_t[4]	100V
#define BMS_CELL_TEMPERATURE(n)			(CANR_FCN_BMSDATA_ID | CANR_GRP_BMS_TEMPERATURE_ID | (n))			// uint16_t[4]	0.1 deg C
#define BMS_VOLTAGE_METADATA				(CANR_FCN_DATA_ID | CANR_GRP_BMS_ID | CANR_MODULE_ID0_ID)		// uint16_t		100 V
#define BMS_TEMPERATURE_METADATA		(CANR_FCN_DATA_ID | CANR_GRP_BMS_ID | CANR_MODULE_ID1_ID)		// uint16_t		0.1 deg C
#define BMS_TS_DATA									(CANR_FCN_DATA_ID | CANR_GRP_BMS_ID | CANR_MODULE_ID2_ID)		// int16_t			100 mV, 100 mA, 100 mW, 0.01%
#define BMS_STATUS_FLAGS						(CANR_FCN_DATA_ID | CANR_GRP_BMS_ID | CANR_MODULE_ID3_ID)		// uint16_t		bitfield (see below)

#define BMS_STATUS_FLAG_OVERVOLTAGE					(1 << 0)
#define BMS_STATUS_FLAG_UNDERVOLTAGE				(1 << 1)
#define BMS_STATUS_FLAG_OVERTEMPERATURE			(1 << 2)
#define BMS_STATUS_FLAG_OVERCURRENT					(1 << 3)
#define BMS_STATUS_FLAG_OVERPOWER						(1 << 4)
#define BMS_STATUS_FLAG_ISOLATIONFAULT				(1 << 5)
#define BMS_STATUS_FLAG_ISOSPI_LOSSOFSIGNAL		(1 << 6)
#define BMS_STATUS_FLAG_CM_LOSSOFSIGNAL			(1 << 7)
#define BMS_STATUS_FLAG_HVM_LOSSOFSIGNAL		(1 << 8)

//-------------------------------------------------------------------------------------



//-------------------------------------------------------------------------------
//                          BMS CAN IDS
//-------------------------------------------------------------------------------
#define GLVBMS_VTG_MSG0_ID							(CANR_FCN_DATA_ID | CANR_GRP_GLVBMS_ID | CANR_MODULE_ID0_ID) 
#define GLVBMS_VGT_MSG1_ID							(CANR_FCN_DATA_ID | CANR_GRP_GLVBMS_ID | CANR_MODULE_ID1_ID)
#define GLVBMS_TMP_MSG0_ID							(CANR_FCN_DATA_ID | CANR_GRP_GLVBMS_ID | CANR_MODULE_ID2_ID)
#define GLVBMS_TMP_MSG1_ID							(CANR_FCN_DATA_ID | CANR_GRP_GLVBMS_ID | CANR_MODULE_ID3_ID)
#define GLVBMS_CURRENT_ID							(CANR_FCN_DATA_ID | CANR_GRP_GLVBMS_ID | CANR_MODULE_ID4_ID)
#define GLVBMS_BATT_STAT_MSG0_ID					(CANR_FCN_DATA_ID | CANR_GRP_GLVBMS_ID | CANR_MODULE_ID5_ID)
#define GLVBMS_BATT_STAT_MSG1_ID					(CANR_FCN_DATA_ID | CANR_GRP_GLVBMS_ID | CANR_MODULE_ID6_ID)

// #define BMS_TOTVTG_ID 1
// #define BMS_MAXMIN_TEMP_ID 2
// #define BMS_MAXMIN_VTG_ID 3
// #define BMS_CURRENT_ID 4
// #define GLVBMS_VTG_ID 6 
// #define GLVBMS_TEMP_ID 7 
// #define GLVBMS_CURRENT_ID 8 
// #define GLVBMS_MAXMIN_VAL_ID 9 
// #define GLVBMS_STATE_MSG_ID 10 
// #define GLVBMS_TOTVTG_ID 11 
// #define GLVBMS_ERROR_MSG_ID 12 

//-------------------------------------------------------------------------------------

//-------------------------------------------------------------------------------------
//                          FAN CONTROL IDS
//-------------------------------------------------------------------------------------
#define FAN_CTRL_STATUS_ID							(CANR_FCN_DATA_ID | CANR_GRP_FAN_CTRL_ID | CANR_MODULE_ID0_ID)
//Byte: 0 = battery fan duty-cycle, 1 = radiator fan duty-cycle, 2 = pump (1=ON,0=OFF), 3 = mode, 4 = brakelight mode, 5 = 12V-state (0=ERROR)
//-------------------------------------------------------------------------------------



#endif /* REVOLVE_CAN_DEFINITIONS_H_ */

