/*
 * enDat.c
 *
 * Created: 22.03.2016 02.03.03
 *  Author: Simen
 */ 

#include <sam.h>
#include <stdbool.h>
#include "../Sam-base/RevolveDrivers/delay.h"
#include "../Sam-base/RevolveDrivers/pio.h"
#include "../Sam-base/RevolveDrivers/pmc.h"
#include "../Sam-base/RevolveDrivers/spi.h"
#include "mazet_endat.h"

#define ENDAT_COMMAND_SEND_POSITION_VALUE							0x07
#define ENDAT_COMMAND_SELECT_MEMORY_AREA							0x0E
#define ENDAT_COMMAND_READ_TEST_VALUES								0x15
#define ENDAT_COMMAND_WRITE_PARAMETERS								0x1C
#define ENDAT_COMMAND_READ_PARAMETERS								0x23
#define ENDAT_COMMAND_RESET_ENCODER									0x2A
#define ENDAT_COMMAND_WRITE_TEST_VALUES								0x31
#define ENDAT_COMMAND_SEND_POSITION_VALUE_AND_ADDITIONAL_DATA		0x38
#define ENDAT_COMMAND_SEND_POSITION_VALUE_AND_SELECTED_MEMORY_AREA	0x09
#define ENDAT_COMMAND_RECIEVE_COMMUNICATION_COMMAND					0x12
#define ENDAT_COMMAND_SEND_POSITION_VALUE_AND_READ_PARAMETERS		0x1B
#define ENDAT_COMMAND_SEND_POSITION_VALUE_AND_WRITE_PARAMETERS		0x24
#define ENDAT_COMMAND_SEND_POSITION_VALUE_AND_RESET					0x2D
#define ENDAT_COMMAND_SEND_POSITION_VALUE_AND_WRITE_TEST_VALUES		0x36		

#define ENDAT_MRS_PARAMETERS			0xA1
#define ENDAT_MRS_WARNINGS_AND_ERRORS	0xB9

#define ENDAT_REG_ERRORS				0x00
#define ENDAT_REG_WARNINGS				0x01
#define ENDAT_REG_BIT_WIDTH				0x0D

void enDat_resetEncoder(void);
void enDat_selectMRS(uint8_t adress);
void enDat_setPositionCommand();
uint32_t enDat_readRegister(uint8_t adress);
void enDat_clearRegister(uint8_t adress);

void enDat_init(){
	__attribute__((unused)) volatile uint32_t errors		= 0;
	__attribute__((unused)) volatile uint32_t warnings		= 0;
	__attribute__((unused)) volatile uint32_t bits			= 0;
	__attribute__((unused)) volatile uint32_t status_bits	= 0;
	
	mazet_init();
	
	delay_ms(900);
	
	enDat_resetEncoder();
		
	//Read errors
	enDat_selectMRS(ENDAT_MRS_WARNINGS_AND_ERRORS);
	errors = enDat_readRegister(ENDAT_REG_ERRORS);
	//Reset pending errors
	enDat_clearRegister(ENDAT_REG_ERRORS);	
	enDat_resetEncoder();
	delay_ms(60);	

	//Read warnings
	enDat_selectMRS(ENDAT_MRS_WARNINGS_AND_ERRORS);
	warnings = enDat_readRegister(ENDAT_REG_WARNINGS);
	//Reset pending warnings
	enDat_clearRegister(ENDAT_REG_WARNINGS);	
	enDat_resetEncoder();

	delay_ms(60);

	//Get data width
	enDat_selectMRS(ENDAT_MRS_PARAMETERS);
	bits = enDat_readRegister(ENDAT_REG_BIT_WIDTH);

	status_bits = mazet_getStatus();
	
	mazet_setEncoderCommand(ENDAT_COMMAND_SEND_POSITION_VALUE, 0, 0);
	mazet_setInterruptMask(MAZET_STATUS_REC_1_UPDATED_bit);
	
	delay_ms(40);
	
	mazet_initDMAMachine();
	mazet_setAutoTrigger(40);
}

void enDat_resetEncoder(void){
	mazet_setEncoderCommand(ENDAT_COMMAND_RESET_ENCODER, 0, 0);
	mazet_strobeSW();
	delay_ms(1);
}

void enDat_selectMRS(uint8_t adress){
	mazet_setEncoderCommand(ENDAT_COMMAND_SELECT_MEMORY_AREA, adress, 0);
	mazet_strobeSW();
	delay_ms(1);
}

uint32_t enDat_readRegister(uint8_t adress){
	mazet_setEncoderCommand(ENDAT_COMMAND_READ_PARAMETERS, adress, 0);
	mazet_strobeSW();
	delay_ms(1);
	return mazet_getRegister(MAZET_REGISTER_ADRESS_RECIEVE_1);
}

void enDat_clearRegister(uint8_t adress){
	mazet_setEncoderCommand(ENDAT_COMMAND_WRITE_PARAMETERS, adress, 0);
	mazet_strobeSW();
	delay_ms(1);
}

void enDat_startNewTransmission(){
	spi_flush_rx_register(SPI0);
	mazet_stopDMAGet();
	mazet_startDMAGet();
}

uint32_t enDat_getLastPositionData(){
	return mazet_getPositionData();
}

uint32_t enDat_getLastStatusData(){
	return mazet_getStatusData();
}