/*
 * mazet_endat.c
 *
 * Created: 10.04.2016 19.17.03
 *  Author: simtind
 */ 

#include <sam.h>
#include <stdbool.h>
#include <string.h>
#include "../Sam-base/RevolveDrivers/delay.h"
#include "../Sam-base/RevolveDrivers/pio.h"
#include "../Sam-base/RevolveDrivers/pmc.h"
#include "../Sam-base/RevolveDrivers/spi.h"
#include "../Sam-base/RevolveDrivers/xdmac_wrapper.h"

#include "mazet_endat.h"

#define MAZET_RESET_PIO PIOA, 1
#define MAZET_STROBE_PIO PIOA, 0
#define MAZET_MODE16_PIO PIOD, 11
#define FPGA_RESET_PIO PIOD, 11

typedef enum {
	enDat_write = 0,
	enDat_read = (1 << 6)
}enDat_operation;

#define MAZET_DMA_CHANNEL_GET_TX	0
#define MAZET_DMA_CHANNEL_GET_RX	1
#define MAZET_DMA_CHANNEL_SET		2
#define MAZET_DMA_TRIGGER_RX XDMAC_EVENT_HWID_SPI0_RX
#define MAZET_DMA_TRIGGER_TX XDMAC_EVENT_HWID_SPI0_TX

void mazet_DMACallback(uint32_t status);

volatile uint32_t tx_buffer_DMAget[6]	= {0};
volatile uint32_t rx_buffer_DMAget[6]	= {0};
volatile uint32_t tx_buffer_DMASet[4]	= {0};
uint32_t mazet_config[3]				= {0};

Xdma_transfer_opts_t mazet_transfer_tx_DMAGet = {
	.peripheral_transfer_dir = XDMAC_DIR_MEM2PER,
	.num_transfers = 0,
	.element_size = XDMAC_SIZE_32,
	.source_adress = tx_buffer_DMAget,
	.channel_number = MAZET_DMA_CHANNEL_GET_TX,
	.trigger = MAZET_DMA_TRIGGER_TX,
	.array_size = 6,
	.destination_bus = XDMAC_BUS_1,
	.source_bus = XDMAC_BUS_0
};

Xdma_transfer_opts_t mazet_transfer_rx_DMAGet = {
	.peripheral_transfer_dir = XDMAC_DIR_PER2MEM,
	.num_transfers = 0,
	.element_size = XDMAC_SIZE_32,
	.destination_adress = rx_buffer_DMAget,
	.channel_number = MAZET_DMA_CHANNEL_GET_RX,
	.trigger = MAZET_DMA_TRIGGER_RX,
	.array_size = 6,
	.interrupt_mask = XDMAC_CIE_BIE,
	.destination_bus = XDMAC_BUS_0,
	.source_bus = XDMAC_BUS_1
};

Xdma_transfer_opts_t mazet_transfer_tx_DMASet = {
	.peripheral_transfer_dir = XDMAC_DIR_MEM2PER,
	.num_transfers = 0,
	.element_size = XDMAC_SIZE_32,
	.source_adress = tx_buffer_DMASet,
	.channel_number = MAZET_DMA_CHANNEL_SET,
	.trigger = MAZET_DMA_TRIGGER_TX,
	.array_size = 3
};
	
uint32_t mazet_getStatus();
void mazet_setRegister_DMA(uint8_t adress, uint32_t data);
void mazet_setRegister(uint8_t adress, uint32_t data);
uint32_t mazet_getRegister(uint8_t adress);
void mazet_getRegister_DMA(uint8_t adress);
uint32_t mazet_getRegister_DMAget();
void mazet_resetDecoder();
void mazet_setEncoderCommand(uint8_t command, uint8_t parameter, uint16_t data);
void mazet_strobeHW();
void mazet_strobeSW();
uint8_t mazet_getIDByte(uint8_t data_length, enDat_operation operation);
bool mazet_isCommandFinished();
uint8_t mazet_selectTransmitPrescaler(Mazet_fsys f_sys, EnDat_fclk f_clk);
void mazet_setConfig();
void mazet_getDefaultConfig(uint32_t configArray[3]);
void mazet_setupDMASet(uint32_t data);
void mazet_setupDMAGet();
void mazet_hardReset();

void mazet_init(){
	struct SpiSlaveSettings enDatMaster_SPISettings = {
		.chip_select = NPCS3,
		.spi_baudRate_Hz = 2000000,
		.data_width = SPI_16BIT_DATA,
		.raise_CS_every_transfer = true,
		.spi_mode = MODE_2
	};
	
	struct SpiMasterSettings enDatMaster_SPIMuxSettings = {
		.cs_3 = PD27,
		.CSDelay_ns = 100,
	};
	
	spi_master_init(SPI0, enDatMaster_SPIMuxSettings);
	spi_chip_select_init(SPI0, enDatMaster_SPISettings);
	
	pio_enableOutput(MAZET_RESET_PIO);
	pio_enableOutput(MAZET_STROBE_PIO);
	pio_enableOutput(MAZET_MODE16_PIO);
	pio_enableOutput(FPGA_RESET_PIO);	
	
	pio_setOutput(MAZET_MODE16_PIO, PIN_HIGH);
	pio_setOutput(MAZET_STROBE_PIO, PIN_HIGH);
	pio_setOutput(FPGA_RESET_PIO, PIN_HIGH);
	
	mazet_hardReset();

	mazet_resetDecoder();

	mazet_getDefaultConfig(mazet_config);
	mazet_setConfig();
	
	volatile uint32_t mazet_config_readback = mazet_getRegister(MAZET_REGISTER_ADRESS_CONFIG_1);
	while(mazet_config_readback != mazet_config[0]){
		mazet_hardReset();
		mazet_resetDecoder();
		mazet_setConfig();
		mazet_config_readback = mazet_getRegister(MAZET_REGISTER_ADRESS_CONFIG_1);
	}
}

void mazet_hardReset(){
	pio_setOutput(MAZET_RESET_PIO, PIN_LOW);	//Hard reset FPGA IP
	delay_us(1);
	pio_setOutput(MAZET_RESET_PIO, PIN_HIGH);
}

void mazet_initDMAMachine(){
	mazet_setupDMASet(MAZET_STATUS_REC_1_UPDATED_bit);
	mazet_setupDMAGet();
	
	xdma_setup_peripheral_transfer(XDMAC, &mazet_transfer_tx_DMAGet);
	xdma_setup_peripheral_transfer(XDMAC, &mazet_transfer_rx_DMAGet);
	//xdma_setup_peripheral_transfer(XDMAC, &mazet_transfer_tx_DMASet);
	
/*
	pio_setPull(PIOA, 03, PULLUP);
	pio_enableInterrupt(PIOA, 03, FALLING_EDGE, mazet_IOCallback );
	
	if(!pio_readPin(PIOA, 03)){
		mazet_IOCallback();
	}*/
}

static const uint8_t endat_tx_prescaler[5][8] = {
	{0x0F, 0x0E, 0x0D, 0x0C, 0x04, 0x03, 0xFF, 0xFF},	//32
	{0x0F, 0x0E, 0x0D, 0x0C, 0x06, 0x0FF, 0x03, 0xFF},	//48
	{0x0F, 0x0E, 0x0D, 0x0C, 0x06, 0x0FF, 0x03, 0xFF},	//50
	{0x0F, 0x0E, 0x0D, 0x0C, 0x08, 0x0FF, 0x04, 0xFF},	//64
	{0x0F, 0x0E, 0x0D, 0x0C, 0x0B, 0x0FF, 0x06, 0x03}	//100
};

void mazet_setInterruptMask(uint32_t interruptMask){
	mazet_setRegister(MAZET_REGISTER_ADRESS_INTERRUPT_MASK, interruptMask); //Enable receive interrupt
}

void mazet_setAutoTrigger(uint16_t period_us){
	mazet_setRegister(MAZET_REGISTER_ADRESS_CONFIG_2, (period_us / 2) << MAZET_CONFIG2_SAMPLING_COUNTER_pos);
}

uint32_t mazet_getStatus(){
	return mazet_getRegister(MAZET_REGISTER_ADRESS_STATUS);
}

void mazet_setRegister(uint8_t adress, uint32_t data){
	uint8_t tx_buffer[6] = {0};
	
	tx_buffer[0] = adress;
	tx_buffer[2] = ((uint8_t *)&data)[2];
	tx_buffer[3] = ((uint8_t *)&data)[3];
	tx_buffer[4] = ((uint8_t *)&data)[0];
	tx_buffer[5] = ((uint8_t *)&data)[1];
	
	if(adress == MAZET_REGISTER_ADRESS_CONFIG_3){
		tx_buffer[1] = mazet_getIDByte(2, enDat_write);
		spi_transceive((uint16_t *)tx_buffer, NULL, 2, NPCS3);
	}
	else{
		tx_buffer[1] = mazet_getIDByte(4, enDat_write);
		spi_transceive((uint16_t *)tx_buffer, NULL, 3, NPCS3);
	}
}

uint32_t mazet_getRegister(uint8_t adress){
	uint8_t	tx_buffer[6] = {0xFF};
	uint16_t rx_buffer[4] = {0};
	
	if(adress == MAZET_REGISTER_ADRESS_CONFIG_3){
		tx_buffer[1] = mazet_getIDByte(2, enDat_read);
		tx_buffer[0] = adress;
		spi_transceive((uint16_t *)tx_buffer, (uint16_t *)rx_buffer, 2, NPCS3);
	}
	else{
		tx_buffer[1] = mazet_getIDByte(4, enDat_read);
		tx_buffer[0] = adress;
		while (!(SPI0->SPI_SR & SPI_SR_TDRE));
		
		spi_transceive((uint16_t *)tx_buffer, rx_buffer, 3, NPCS3);
	}
	return ((rx_buffer[1] << 16) | rx_buffer[2]);
}

void mazet_setupDMASet(uint32_t data){	
	mazet_transfer_tx_DMASet.destination_adress = spi_get_tx_register_address(SPI0);
	
	mazet_transfer_tx_DMASet.source_adress[0] = spi_generateControlWord(false, NPCS3, (mazet_getIDByte(4, enDat_write) << 8) | MAZET_REGISTER_ADRESS_STATUS);
	mazet_transfer_tx_DMASet.source_adress[1] = spi_generateControlWord(false, NPCS3, ((data & (0xFFFF << 16)) >> 16));
	mazet_transfer_tx_DMASet.source_adress[2] = spi_generateControlWord(true, NPCS3, (data & (0xFFFF)));
}

void mazet_setupDMAGet(){	
	mazet_transfer_tx_DMAGet.destination_adress = spi_get_tx_register_address(SPI0);
	mazet_transfer_rx_DMAGet.source_adress = spi_get_rx_register_address(SPI0);

	mazet_transfer_tx_DMAGet.source_adress[0] = spi_generateControlWord(false, NPCS3, (mazet_getIDByte(4, enDat_read) << 8) |  MAZET_REGISTER_ADRESS_RECIEVE_1 );
	mazet_transfer_tx_DMAGet.source_adress[1] = spi_generateControlWord(false, NPCS3, 0);
	mazet_transfer_tx_DMAGet.source_adress[2] = spi_generateControlWord(true, NPCS3, 0);
	
	mazet_transfer_tx_DMAGet.source_adress[3] = spi_generateControlWord(false, NPCS3, (mazet_getIDByte(4, enDat_read) << 8) |  MAZET_REGISTER_ADRESS_STATUS );
	mazet_transfer_tx_DMAGet.source_adress[4] = spi_generateControlWord(false, NPCS3, 0);
	mazet_transfer_tx_DMAGet.source_adress[5] = spi_generateControlWord(true, NPCS3, 0);
	
	mazet_transfer_rx_DMAGet.destination_adress[0] = 0;
	mazet_transfer_rx_DMAGet.destination_adress[1] = 0;
	mazet_transfer_rx_DMAGet.destination_adress[2] = 0;
	mazet_transfer_rx_DMAGet.destination_adress[3] = 0;
	mazet_transfer_rx_DMAGet.destination_adress[4] = 0;
	mazet_transfer_rx_DMAGet.destination_adress[5] = 0;
}

void mazet_startDMAGet(){
	xdma_start_transfer(XDMAC, &mazet_transfer_tx_DMAGet);
	xdma_start_transfer(XDMAC, &mazet_transfer_rx_DMAGet);
	//SPI0->SPI_TDR = mazet_transfer_tx_DMAGet.source_adress[-1];
}

void mazet_stopDMAGet(){
	xdma_stop_transfer(XDMAC, &mazet_transfer_tx_DMAGet);
	xdma_stop_transfer(XDMAC, &mazet_transfer_rx_DMAGet);
}

void mazet_startDMASet(){
	xdma_start_transfer(XDMAC, &mazet_transfer_tx_DMASet);
	//SPI0->SPI_TDR = mazet_transfer_tx_DMASet.source_adress[-1];
}

void mazet_stopDMASet(){
	xdma_stop_transfer(XDMAC, &mazet_transfer_tx_DMASet);
}

void mazet_resetDecoder(){
	mazet_setRegister(MAZET_REGISTER_ADRESS_CONFIG_1, MAZET_CONFIG1_RESET_bit);
}

void mazet_setEncoderCommand(uint8_t command, uint8_t parameter, uint16_t data){
	mazet_setRegister(MAZET_REGISTER_ADRESS_SEND, (command << 24) | (parameter << 16) | (data));
}

void mazet_strobeHW(){
	pio_setOutput(PIOA,  00, PIN_LOW);
	pio_setOutput(PIOA,  00, PIN_HIGH);
}

uint32_t mazet_getPositionData(){
	return (((mazet_transfer_rx_DMAGet.destination_adress[1] & 0x03) << 16) | (mazet_transfer_rx_DMAGet.destination_adress[2] & 0xFFFF));
}

uint32_t mazet_getStatusData(){
	return (((mazet_transfer_rx_DMAGet.destination_adress[4] & 0xFFFF) << 16) | (mazet_transfer_rx_DMAGet.destination_adress[5] & 0xFFFF));
}

void mazet_strobeSW(){
	mazet_setRegister(MAZET_REGISTER_ADRESS_SW_STROBE, 0xFFFFFFFF);
}


uint8_t mazet_getIDByte(uint8_t data_length, enDat_operation operation){
	return (operation | data_length) & (~(1 << 7));
}

bool mazet_isCommandFinished(){
	uint32_t res = mazet_getRegister(MAZET_REGISTER_ADRESS_STATUS);
	return res & MAZET_STATUS_READY_bit;
}


uint8_t mazet_selectTransmitPrescaler(Mazet_fsys f_sys, EnDat_fclk f_clk){
	return endat_tx_prescaler[f_sys][f_clk];
}

void mazet_setConfig(){
	mazet_setRegister(MAZET_REGISTER_ADRESS_CONFIG_1, mazet_config[0]);
	mazet_setRegister(MAZET_REGISTER_ADRESS_CONFIG_2, mazet_config[1]);
	mazet_setRegister(MAZET_REGISTER_ADRESS_CONFIG_3, mazet_config[2]);
}

void mazet_getDefaultConfig(uint32_t configArray[3]){
	configArray[0] =	MAZET_CONFIG1_HW_STROBE_en_bit					|
						MAZET_CONFIG1_FORCE_RECIEVE_en_bit				|
						MAZET_CONFIG1_TRANSMIT_CLK_PRESCALER(mazet_selectTransmitPrescaler(MAZET_FSYS_64MHz, ENDAT_FCLK_2MHz))|
						MAZET_CONFIG1_DATA_LENGTH(18)					|
						MAZET_CONFIG1_DELAY_COMP_en_bit					|
						MAZET_CONFIG1_SYSTEM_CLOCK_SET_64MHz			|
						MAZET_CONFIG1_PROTOCOL_SET_ENDAT				;
	configArray[1] =	0;//MAZET_CONFIG2_RECOVERY_TIME(MAZET_RECOVERY_TIME_2us);
	configArray[2] =	0;
}
