/*
 * mazet_endat.h
 *
 * Created: 10.04.2016 19.17.19
 *  Author: simtind
 */ 


#ifndef MAZET_ENDAT_H_
#define MAZET_ENDAT_H_

#include "mazet_status.h"

#define MAZET_REGISTER_ADRESS_SEND				0x00	//
#define MAZET_REGISTER_ADRESS_RECIEVE_1			0x04	//
#define MAZET_REGISTER_ADRESS_RECIEVE_2			0x0C	//
#define MAZET_REGISTER_ADRESS_RECIEVE_3			0x10	//
#define MAZET_REGISTER_ADRESS_CONFIG_1			0x14	//Basic settings and EnDat functions
#define MAZET_REGISTER_ADRESS_CONFIG_2			0x18	//Protocol-independent settings
#define MAZET_REGISTER_ADRESS_CONFIG_3			0x1C	//SSI functions
#define MAZET_REGISTER_ADRESS_STATUS			0x20	//
#define MAZET_REGISTER_ADRESS_INTERRUPT_MASK	0x24	//
#define MAZET_REGISTER_ADRESS_TEST_1			0x28	//
#define MAZET_REGISTER_ADRESS_TEST_2			0x2C	//
#define MAZET_REGISTER_ADRESS_RECIEVE_4			0x30	//
#define MAZET_REGISTER_ADRESS_SW_STROBE			0x38	//
#define MAZET_REGISTER_ADRESS_ID				0x3C	//

#define MAZET_CONFIG1_HW_STROBE_en_bit			(1 << 0)
#define MAZET_CONFIG1_FORCE_RECIEVE_en_bit		(1 << 1)
#define MAZET_CONFIG1_CONTINUOUS_CLOCK_en_bit	(1 << 2)
#define MAZET_CONFIG1_TRANSMIT_CLK_PRESCALER(p)	(p << 4)
#define MAZET_CONFIG1_DATA_LENGTH(length)		(length << 8)
#define MAZET_CONFIG1_RESET_WINDOW_en_bit		(1 << 14)
#define MAZET_CONFIG1_AUTO_RESET_en_bit			(1 << 15)
#define MAZET_CONFIG1_CABLE_PROPAGATION_pos		16
#define MAZET_CONFIG1_DELAY_COMP_en_bit			(1 << 24)
#define MAZET_CONFIG1_SYSTEM_CLOCK_SET_64MHz	(0 << 26)
#define MAZET_CONFIG1_SYSTEM_CLOCK_SET_48MHz	(2 << 26)
#define MAZET_CONFIG1_SYSTEM_CLOCK_SET_32MHz	(4 << 26)
#define MAZET_CONFIG1_SYSTEM_CLOCK_SET_50MHz	(5 << 26)
#define MAZET_CONFIG1_SYSTEM_CLOCK_SET_100MHz	(6 << 26)
#define MAZET_CONFIG1_RESET_bit					(1 << 29)
#define MAZET_CONFIG1_PROTOCOL_SET_ENDAT		(0x2 << 30)
#define MAZET_CONFIG1_PROTOCOL_SET_SSI			(0x1 << 30)


#define MAZET_CONFIG2_SAMPLING_COUNTER_pos		0
#define MAZET_CONFIG2_WATCHDOG_pos				8
#define MAZET_CONFIG2_RECOVERY_TIME(t)			(t <<  16)
#define MAZET_CONFIG2_FILTER_pos				19
#define MAZET_CONFIG2_STROBE_DELAY_pos			24

#define MAZET_RECOVERY_TIME_0_5TCLK				0
#define MAZET_RECOVERY_TIME_0_5_us				1
#define MAZET_RECOVERY_TIME_1us					2
#define MAZET_RECOVERY_TIME_1_5us				3
#define MAZET_RECOVERY_TIME_2us					4
#define MAZET_RECOVERY_TIME_4us					5
#define MAZET_RECOVERY_TIME_8us					6
#define MAZET_RECOVERY_TIME_10us				7

#define MAZET_CONFIG3_PARITY_pos				0
#define MAZET_CONFIG3_FORMAT_pos				1
#define MAZET_CONFIG3_GRAY_TO_BIN_pos			2
#define MAZET_CONFIG3_RESOLUTION_pos			3
#define MAZET_CONFIG3_DOUBLEWORD_pos			8
#define MAZET_CONFIG3_VELOCITY_WIDTH_pos		15



typedef enum {
	MAZET_FSYS_32MHz,
	MAZET_FSYS_48MHz,
	MAZET_FSYS_50MHz,
	MAZET_FSYS_64MHz,
	MAZET_FSYS_100MHz
}Mazet_fsys;

typedef enum {
	ENDAT_FCLK_100kHz,
	ENDAT_FCLK_200kHz,
	ENDAT_FCLK_1MHz,
	ENDAT_FCLK_2MHz,
	ENDAT_FCLK_4MHz,
	ENDAT_FCLK_5MHz,
	ENDAT_FCLK_8MHz,
	ENDAT_FCLK_16MHz
}EnDat_fclk;


void mazet_init();
void mazet_initDMAMachine();
void mazet_setInterruptMask(uint32_t interruptMask);
void mazet_setAutoTrigger(uint16_t period_us);

uint32_t mazet_getStatus();
void mazet_setRegister(uint8_t adress, uint32_t data);
uint32_t mazet_getRegister(uint8_t adress);

void mazet_setRegister_DMA(uint8_t adress, uint32_t data);
void mazet_getRegister_DMA(uint8_t adress);

void mazet_setEncoderCommand(uint8_t command, uint8_t parameter, uint16_t data);
bool mazet_isCommandFinished();

uint32_t mazet_getPositionData();
uint32_t mazet_getStatusData();

void mazet_strobeHW();
void mazet_strobeSW();

void mazet_startDMAGet();
void mazet_stopDMAGet();

#endif /* MAZET_ENDAT_H_ */