/*
 * phase_voltage.c
 *
 * Created: 01.03.2016 16.57.28
 *  Author: Simen
 */ 

#include <sam.h>
#include <arm_math.h>
#include <stdbool.h>
#include "phase_voltage.h"
#include "../Sam-base/RevolveDrivers/pio.h"
#include "../Sam-base/RevolveDrivers/PWM/pwm.h"
#include "voltageModulation.h"
#define PWM_DEADTIME_NS 4000
#define PWM_FREQUENCY_HZ 8000

pwm_opt_t pwm_settings = {
	.active_high_out = false,
	.center_aligned = true,
	.deadtime_enabled = true,
	.enable_period_start_event = true
};

void phase_voltage_init(){
	pwm_settings.deadtime_pwmh_ticks	= pwm_calc_deadtime_ticks(PWM_DEADTIME_NS);
	pwm_settings.deadtime_pwml_ticks	= pwm_calc_deadtime_ticks(PWM_DEADTIME_NS);
	pwm_settings.pwm_period_ticks		= pwm_calc_period_ticks(PWM_FREQUENCY_HZ, pwm_settings.center_aligned) ;
	
	pwm_init(PWM0, &pwm_settings, 4);
	/*pio_setMux(PIOA, 00, A);
	pio_setMux(PIOA, 19, B);
	
	pio_setMux(PIOD, 21, A);	//PWMH1
	pio_setMux(PIOD, 25, A);	//PWML1
	
	pio_setMux(PIOC, 19, B);	//PWMH2
	pio_setMux(PIOD, 26, A);	//PWML2
	
	pio_setMux(PIOD, 11, B);
	pio_setMux(PIOD, 24, A);*/
	
	pio_setMux(PIOA, 24, B);	//PWMH1
	pio_setMux(PIOD, 25, A);	//PWML1
	
	pio_setMux(PIOA, 13, B);	//PWMH2
	pio_setMux(PIOA, 16, C);	//PWML2
	
	pio_setMux(PIOA, 14, B);	//PWMH3
	pio_setMux(PIOA, 15, C);	//PWML3
	
}

void phase_voltage_disable_phase(phase_type_t phase){
	pwm_enable_override(PWM0, phase);
}

void phase_voltage_enable_phase(phase_type_t phase){
	pwm_disable_override(PWM0, phase);
}

void phase_voltage_set_fraction(phase_type_t phase, float32_t phase_voltage){
	if(phase == PHASE_ALL){
		pwm_update_channel_data(PWM0, &pwm_settings, PHASE_A, pwm_settings.pwm_period_ticks * (phase_voltage+0.5));
		pwm_update_channel_data(PWM0, &pwm_settings, PHASE_B, pwm_settings.pwm_period_ticks * (phase_voltage+0.5));
		pwm_update_channel_data(PWM0, &pwm_settings, PHASE_C, pwm_settings.pwm_period_ticks * (phase_voltage+0.5));
	}
	else{
		pwm_update_channel_data(PWM0, &pwm_settings, phase, pwm_settings.pwm_period_ticks * (phase_voltage+0.5));
	}
}

void phase_voltage_set_voltage(phase_type_t phase, float32_t phase_voltage, float32_t dc_voltage){
	if(phase == PHASE_ALL){
		pwm_update_channel_data(PWM0, &pwm_settings, PHASE_A, pwm_settings.pwm_period_ticks * voltageModulation_voltageToPwm(phase_voltage, dc_voltage, 0.0, 1.0));
		pwm_update_channel_data(PWM0, &pwm_settings, PHASE_B, pwm_settings.pwm_period_ticks * voltageModulation_voltageToPwm(phase_voltage, dc_voltage, 0.0, 1.0));
		pwm_update_channel_data(PWM0, &pwm_settings, PHASE_C, pwm_settings.pwm_period_ticks * voltageModulation_voltageToPwm(phase_voltage, dc_voltage, 0.0, 1.0));
	}
	else{
		pwm_update_channel_data(PWM0, &pwm_settings, phase, pwm_settings.pwm_period_ticks * voltageModulation_voltageToPwm(phase_voltage, dc_voltage, 0.0, 1.0));
	}
}