#ifndef MCAN_H_
#define MCAN_H_

#include <sam.h>
#include "can_status_codes.h"

#include <stdbool.h>

#define MCAN0_RX_FIFO_0_NUM					60	// Max 64
#define MCAN0_TX_FIFO_QUEUE_NUM				32	// Max 32 
#define MCAN0_RX_STANDARD_ID_FILTER_NUM     120
#define MCAN0_RX_EXTENDED_ID_FILTER_NUM		10

#define MCAN1_RX_FIFO_0_NUM					60   // Max 64           
#define MCAN1_TX_FIFO_QUEUE_NUM				32   // Max 32   
#define MCAN1_RX_STANDARD_ID_FILTER_NUM     120 
#define MCAN1_RX_EXTENDED_ID_FILTER_NUM		10

#define MCAN_ELEMENT_DATA_SIZE				8



//*********************************************************************
// ASF ASF ASF ASF ASF ASF ASF ASFASF ASF ASF ASFASF ASF ASF ASF ASF 
//*********************************************************************

typedef union {
	struct {
		/* bit:  0..28  Identifier */
		uint32_t ID:29;
		/* bit:  29     Remote Transmission Request */
		uint32_t RTR:1;
		/* bit:  30     Extended Identifier */
		uint32_t XTD:1;
		/* bit:  31     Error State Indicator */
		uint32_t ESI:1;
	} bit;
	/* Type used for register access */
	uint32_t reg;
} MCAN_RX_ELEMENT_R0_Type;

typedef union {
	struct {
		/* bit: 0..15   Rx Timestamp */
		uint32_t RXTS:16;
		/* bit: 16..19  Data Length Code */
		uint32_t DLC:4;
		/* bit: 20      Bit Rate Switch */
		uint32_t BRS:1;
		/* bit: 21      FD Format */
		uint32_t EDL:1;
		/* bit: 22..23  Reserved */
		uint32_t :2;
		/* bit: 24..30  Filter Index */
		uint32_t FIDX:7;
		/* bit: 31      Accepted Non-matching Frame */
		uint32_t ANMF:1;
	} bit;
	 /* Type used for register access */
	uint32_t reg;
} MCAN_RX_ELEMENT_R1_Type;

struct mcan_rx_element_fifo_0 {
	__IO MCAN_RX_ELEMENT_R0_Type R0;
	__IO MCAN_RX_ELEMENT_R1_Type R1;
	uint8_t data[MCAN_ELEMENT_DATA_SIZE];
};
typedef union {
	struct {
		/* bit:  0..28  Identifier */
		uint32_t ID:29;
		/* bit:  29     Remote Transmission Request */
		uint32_t RTR:1;
		/* bit:  30     Extended Identifier */
		uint32_t XTD:1;
		/* bit:  31     Reserved */
		uint32_t :1;
	} bit;
	/* Type used for register access */
  uint32_t reg;
} MCAN_TX_ELEMENT_T0_Type;


#define MCAN_TX_ELEMENT_T0_STANDARD_ID_Pos          18
#define MCAN_TX_ELEMENT_T0_STANDARD_ID_Msk          (0x7FFul << MCAN_TX_ELEMENT_T0_STANDARD_ID_Pos)




struct mcan_rx_element_fifo_1 {
	__IO MCAN_RX_ELEMENT_R0_Type R0;
	__IO MCAN_RX_ELEMENT_R1_Type R1;
	uint8_t data[MCAN_ELEMENT_DATA_SIZE];
};

typedef union {
	struct {
		/* bit: 0..15   Reserved */
		uint32_t :16;
		/* bit: 16..19  Data Length Code */
		uint32_t DLC:4;
		/* bit: 20..22  Reserved */
		uint32_t :3;
		/* bit: 23      Event FIFO Control */
		uint32_t EFCC:1;
		/* bit: 24..31  Message Marker */
		uint32_t MM:8;
	} bit;
	/* Type used for register access */
	uint32_t reg;
} MCAN_TX_ELEMENT_T1_Type;

struct mcan_tx_element {
	__IO MCAN_TX_ELEMENT_T0_Type T0;
	__IO MCAN_TX_ELEMENT_T1_Type T1;
	uint8_t data[MCAN_ELEMENT_DATA_SIZE];
};



typedef union {
	struct {
		/* bit: 0..10   Standard Filter ID 2 */
		uint32_t SFID2:11;
		/* bit: 11..15  Reserved */
		uint32_t :5;
		/* bit: 16..26  Standard Filter ID 1 */
		uint32_t SFID1:11;
		/* bit: 27..29  Standard Filter Element Configuration */
		uint32_t SFEC:3;
		/* bit: 30..31  Standard Filter Type */
		uint32_t SFT:2;
	} bit;
	/* Type used for register access */
	uint32_t reg;
} MCAN_STANDARD_MESSAGE_FILTER_ELEMENT_S0_Type;


struct mcan_standard_message_filter_element {
	__IO MCAN_STANDARD_MESSAGE_FILTER_ELEMENT_S0_Type S0;
};

// ************** EXTENDED ***************** //
typedef union {
	struct {
		/* bit: 0..28   Extended Filter ID 1 */
		uint32_t EFID1:29;
		/* bit: 29..31  Extended Filter Element Configuration */
		uint32_t EFEC:3;
	} bit;
	/* Type used for register access */
	uint32_t reg;
} MCAN_EXTENDED_MESSAGE_FILTER_ELEMENT_F0_Type;

typedef union {
	struct {
		/* bit: 0..28  Extended Filter ID 2 */
		uint32_t EFID2:29;
		/* bit: 29     Reserved */
		uint32_t :1;
		/* bit: 30..31 Extended Filter Type */
		uint32_t EFT:2;
	} bit;
	/* Type used for register access */
	uint32_t reg;
} MCAN_EXTENDED_MESSAGE_FILTER_ELEMENT_F1_Type;

struct mcan_extended_message_filter_element {
	__IO MCAN_EXTENDED_MESSAGE_FILTER_ELEMENT_F0_Type F0;
	__IO MCAN_EXTENDED_MESSAGE_FILTER_ELEMENT_F1_Type F1;
};







//Can time out modes.
enum mcan_timeout_mode {
	/** Continuous operation. */
	MCAN_TIMEOUT_CONTINUES = MCAN_TOCC_TOS_CONTINUOUS,
	/** Timeout controlled by TX Event FIFO. */
	MCAN_TIMEOUT_TX_EVEN_FIFO = MCAN_TOCC_TOS_TX_EV_TIMEOUT,
	/** Timeout controlled by Rx FIFO 0. */
	MCAN_TIMEOUT_RX_FIFO_0 = MCAN_TOCC_TOS_RX0_EV_TIMEOUT,
	/** Timeout controlled by Rx FIFO 1. */
	MCAN_TIMEOUT_RX_FIFO_1 = MCAN_TOCC_TOS_RX1_EV_TIMEOUT,
};
//Can nonmatching frames action.
enum mcan_nonmatching_frames_action {
	/** Accept in Rx FIFO 0. */
	MCAN_NONMATCHING_FRAMES_FIFO_0,
	/** Accept in Rx FIFO 1. */
	MCAN_NONMATCHING_FRAMES_FIFO_1,
	/** Reject. */
	MCAN_NONMATCHING_FRAMES_REJECT,
};


struct mcan_config {
	/** MCAN run in standby control. */
	bool run_in_standby;
	/** Start value of the Message RAM Watchdog Counter */
	uint8_t watchdog_configuration;
	/** Transmit Pause. */
	bool transmit_pause;
	/** Edge Filtering during Bus Integration. */
	bool edge_filtering;
	/** Protocol Exception Handling. */
	bool protocol_exception_handling;
	/** Automatic Retransmission. */
	bool automatic_retransmission;
	/** Clock Stop Request. */
	bool clock_stop_request;
	/** Clock Stop Acknowledge. */
	bool clock_stop_acknowledge;
	/** Timestamp Counter Prescaler: 0x0-0xF */
	uint8_t timestamp_prescaler;
	/** Timeout Period. */
	uint16_t timeout_period;
	/** Timeout Mode. */
	enum mcan_timeout_mode timeout_mode;
	/** Timeout enable. */
	bool timeout_enable;
	/** Transmitter Delay Compensation Offset : 0x0-0x7F */
	uint8_t delay_compensation_offset;
	/** Transmitter Delay Compensation Filter Window Length : 0x0-0x7F */
	uint8_t delay_compensation_filter_window_length;
	/** Nonmatching frames action for standard frames. */
	enum mcan_nonmatching_frames_action nonmatching_frames_action_standard;
	/** Nonmatching frames action for extended frames. */
	enum mcan_nonmatching_frames_action nonmatching_frames_action_extended;
	/** Reject Remote Standard Frames. */
	bool remote_frames_standard_reject;
	/** Reject Remote Extended Frames. */
	bool remote_frames_extended_reject;
	/** Extended ID Mask: 0x0-0x1FFFFFFF. */
	uint32_t extended_id_mask;
	/** Rx FIFO 0 Operation Mode. */
	bool rx_fifo_0_overwrite;
	/** Rx FIFO 0 Watermark: 1-64, other value disable it. */
	uint8_t rx_fifo_0_watermark;
	/** Rx FIFO 1 Operation Mode. */
	bool rx_fifo_1_overwrite;
	/** Rx FIFO 1 Watermark: 1-64, other value disable it. */
	uint8_t rx_fifo_1_watermark;
	/** Tx FIFO/Queue Mode, 0 for FIFO and 1 for Queue. */
	bool tx_queue_mode;
	/** Tx Event FIFO Watermark: 1-32, other value disable it. */
	uint8_t tx_event_fifo_watermark;
};

static inline void mcan_get_config_defaults(
		struct mcan_config *const config)
{
	/* Default configuration values */
	config->run_in_standby = false;
	config->watchdog_configuration = 0x00;
	config->transmit_pause = true;
	config->edge_filtering = true;
	config->protocol_exception_handling = true;
	config->automatic_retransmission = true;
	config->clock_stop_request = false;
	config->clock_stop_acknowledge = false;
	config->timestamp_prescaler = 0;
	config->timeout_period = 0xFFFF;
	config->timeout_mode = MCAN_TIMEOUT_CONTINUES;
	config->timeout_enable = false;
	config->delay_compensation_offset = 0;
	config->delay_compensation_filter_window_length = 0;
	config->nonmatching_frames_action_standard = MCAN_NONMATCHING_FRAMES_REJECT;
	config->nonmatching_frames_action_extended = MCAN_NONMATCHING_FRAMES_REJECT;
	config->remote_frames_standard_reject = true;
	config->remote_frames_extended_reject = true;
	config->extended_id_mask = 0x1FFFFFFF;
	config->rx_fifo_0_overwrite = true;
	config->rx_fifo_0_watermark = 0;
	config->rx_fifo_1_overwrite = true;
	config->rx_fifo_1_watermark = 0;
	config->tx_queue_mode = false;
	config->tx_event_fifo_watermark = 0;
}


//*******************************************************************************************//

// Counter for fifo 0 index
extern volatile uint32_t mcan0_fifo0_index;
extern volatile uint32_t mcan1_fifo0_index;

void mcan_init(Mcan *can, uint32_t baudRate);


void mcan_setStandardFilter(Mcan *mcan, uint32_t *standardIdList, uint32_t nr_Ids);
void mcan_setExtendedFilter(Mcan *mcan, uint32_t *extendedIdList, uint32_t nr_Ids);
enum status_code mcan_sendMessage(Mcan *mcan, struct Can_message_t *message);
void mcan_getMessage(Mcan *mcan, struct Can_message_t *message, uint32_t index);


void mcan_enable_interrupt(Mcan *mcan, void (*callback_function)(uint32_t index), uint32_t interrupt_priority);
void mcan_disable_interrupt(Mcan *mcan);



/**
 * \brief Set Rx acknowledge.
 *
 * \param[in] module_inst  Pointer to the MCAN software instance struct
 * \param[in] fifo_number  Rx FIFO 0 or 1
 * \param[in] index  Index offset in FIFO
 */


//Enum for the interrupt source.
enum mcan_interrupt_source {
	/** Rx FIFO 0 New Message Interrupt Enable. */
	MCAN_RX_FIFO_0_NEW_MESSAGE = MCAN_IE_RF0NE,
	/** Rx FIFO 0 Watermark Reached Interrupt Enable. */
	MCAN_RX_FIFO_0_WATERMARK = MCAN_IE_RF0WE,
	/** Rx FIFO 0 Full Interrupt Enable. */
	MCAN_RX_FIFO_0_FULL = MCAN_IE_RF0FE,
	/** Rx FIFO 0 Message Lost Interrupt Enable. */
	MCAN_RX_FIFO_0_LOST_MESSAGE = MCAN_IE_RF0LE,
	/** Rx FIFO 1 New Message Interrupt Enable. */
	MCAN_RX_FIFO_1_NEW_MESSAGE = MCAN_IE_RF1NE,
	/** Rx FIFO 1 Watermark Reached Interrupt Enable. */
	MCAN_RX_FIFO_1_WATERMARK = MCAN_IE_RF1WE,
	/** Rx FIFO 1 Full Interrupt Enable. */
	MCAN_RX_FIFO_1_FULL = MCAN_IE_RF1FE,
	/** Rx FIFO 1 Message Lost Interrupt Enable. */
	MCAN_RX_FIFO_1_MESSAGE_LOST = MCAN_IE_RF1LE,
	/** High Priority Message Interrupt Enable. */
	MCAN_RX_HIGH_PRIORITY_MESSAGE = MCAN_IE_HPME,
	/**  Transmission Completed Interrupt Enable. */
	MCAN_TIMESTAMP_COMPLETE = MCAN_IE_TCE,
	/** Transmission Cancellation Finished Interrupt Enable. */
	MCAN_TX_CANCELLATION_FINISH = MCAN_IE_TCFE,
	/** Tx FIFO Empty Interrupt Enable. */
	MCAN_TX_FIFO_EMPTY = MCAN_IE_TFEE,
	/** Tx Event FIFO New Entry Interrupt Enable. */
	MCAN_TX_EVENT_FIFO_NEW_ENTRY = MCAN_IE_TEFNE,
	/** Tx Event FIFO Watermark Reached Interrupt Enable. */
	MCAN_TX_EVENT_FIFO_WATERMARK = MCAN_IE_TEFWE,
	/** Tx Event FIFO Full Interrupt Enable. */
	MCAN_TX_EVENT_FIFO_FULL = MCAN_IE_TEFFE,
	/** Tx Event FIFO Element Lost Interrupt Enable. */
	MCAN_TX_EVENT_FIFO_ELEMENT_LOST = MCAN_IE_TEFLE,
	/** Timestamp Wraparound Interrupt Enable. */
	MCAN_TIMESTAMP_WRAPAROUND = MCAN_IE_TSWE,
	/** Message RAM Access Failure Interrupt Enable. */
	MCAN_MESSAGE_RAM_ACCESS_FAILURE = MCAN_IE_MRAFE,
	/** Timeout Occurred Interrupt Enable. */
	MCAN_TIMEOUT_OCCURRED = MCAN_IE_TOOE,
	/** Message stored to Dedicated Rx Buffer Interrupt Enable. */
	MCAN_RX_BUFFER_NEW_MESSAGE = MCAN_IE_DRXE,
	/** Error Logging Overflow Interrupt Enable. */
	MCAN_ERROR_LOGGING_OVERFLOW = MCAN_IE_ELOE,
	/** Error Passive Interrupt Enable. */
	MCAN_ERROR_PASSIVE = MCAN_IE_EPE,
	/** Warning Status Interrupt Enable. */
	MCAN_WARNING_STATUS = MCAN_IE_EWE,
	/** Bus_Off Status Interrupt Enable. */
	MCAN_BUS_OFF = MCAN_IE_BOE,
	/** Watchdog Interrupt  Enable. */
	MCAN_WATCHDOG = MCAN_IE_WDIE,
	/**CRC Error Interrupt Enable */
	MCAN_CRC_ERROR = MCAN_IE_CRCEE,
	/** Bit Error Interrupt  Enable. */
	MCAN_BIT_ERROR = MCAN_IE_BEE,
	/** Acknowledge Error Interrupt Enable . */
	MCAN_ACKNOWLEDGE_ERROR = MCAN_IE_ACKEE,
	/** Format Error Interrupt Enable */
	MCAN_FORMAT_ERROR = MCAN_IE_FOEE,
	/** Stuff Error Interrupt Enable */
	MCAN_STUFF_ERROR = MCAN_IE_STEE
};

#endif /* MCAN_H_ */