/*
 * flash_userPage.c
 *
 * Created: 15.02.2016 14.12.55
 *  Author: Simen
 */ 

#include <sam.h>
#include <string.h>
#include "flash_userPage.h"
#include "eefc.h"



/*! \name Flash erase page num in FARG[1:0] 
  \note The erase page commands should be cautiouly used as EPA4/EPA32 will not
  take effect according to errata and EPA8/EPA16 must module 8/16 page addresses.*/
//! @{
typedef enum flash_farg_page_num {
	/* 4 of pages to be erased with EPA command*/
	IFLASH_ERASE_PAGES_4=0,
	/* 8 of pages to be erased with EPA command*/
	IFLASH_ERASE_PAGES_8,
	/* 16 of pages to be erased with EPA command*/
	IFLASH_ERASE_PAGES_16,
	/* 32 of pages to be erased with EPA command*/
	IFLASH_ERASE_PAGES_32,
	/* Parameter is not support */
	IFLASH_ERASE_PAGES_INVALID,
}flash_farg_page_num_t;

/* User signature size */
# define FLASH_USER_SIG_SIZE   (512)

/* Flash page buffer for alignment */
static uint32_t gs_ul_page_buffer[IFLASH_PAGE_SIZE / sizeof(uint32_t)];

uint32_t flash_read_user_signature(flash_data_t *data, size_t size)
{
	if (size > FLASH_USER_SIG_SIZE ) {
		/* Only 512 byte to store user signature */
		size = FLASH_USER_SIG_SIZE;
	}

	/* Send the read user signature commands */
	if (FLASH_RC_OK != eefc_perform_read_sequence(EFC, EEFC_FCR_FCMD_STUS,
			EEFC_FCR_FCMD_SPUS, (uint32_t *)data, size)) {
		return FLASH_RC_ERROR;
	}

	return FLASH_RC_OK;
}

uint32_t flash_write_user_signature(const flash_data_t *data, size_t size)
{
	uint32_t ul_idx;
	uint32_t *p_dest;
	volatile uint32_t status;

	/* The user signature should be no longer than 512 bytes */
	if (size > IFLASH_PAGE_SIZE ) {
		return FLASH_RC_INVALID;
	}

	memset((uint8_t *)gs_ul_page_buffer, 0xFFFF, IFLASH_PAGE_SIZE/sizeof(int));

	/* Copy Buffer data */
	memcpy((uint8_t *) gs_ul_page_buffer, data, 
			size * sizeof(uint32_t));

	/* Write page buffer.
	* Writing 8-bit and 16-bit data is not allowed and may lead to
	* unpredictable data corruption.
	*/
	p_dest = (uint32_t *)IFLASH_ADDR;
	for (ul_idx = 0; ul_idx < (IFLASH_PAGE_SIZE / sizeof(uint32_t)); 
			ul_idx++) {
		*p_dest++ = gs_ul_page_buffer[ul_idx];
	}
	
	status = eefc_run_command_and_wait(EFC, EEFC_FCR_FCMD_WUS, 0);

	/* Send the write signature command */
	if (status != FLASH_RC_OK) {
		return FLASH_RC_ERROR;
	}

	return FLASH_RC_OK;
}

uint32_t flash_erase_user_signature(void)
{
	/* Perform the erase user signature command */
	return eefc_run_command_and_wait(EFC, EEFC_FCR_FCMD_EUS, 0);
}