/**
 * \file
 *
 * \brief Pulse Width Modulation (PWM) driver for SAM.
 *
 * Copyright (c) 2011-2015 Atmel Corporation. All rights reserved.
 *
 * \asf_license_start
 *
 * \page License
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 *
 * 3. The name of Atmel may not be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * 4. This software may only be redistributed and used in connection with an
 *    Atmel microcontroller product.
 *
 * THIS SOFTWARE IS PROVIDED BY ATMEL "AS IS" AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT ARE
 * EXPRESSLY AND SPECIFICALLY DISCLAIMED. IN NO EVENT SHALL ATMEL BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
 * ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * \asf_license_stop
 *
 */
/*
 * Support and FAQ: visit <a href="http://www.atmel.com/design-support/">Atmel Support</a>
 */

#ifndef PWM_H_INCLUDED
#define PWM_H_INCLUDED

#include <sam.h>
#include <stdbool.h>

#define PWM_INVALID_ARGUMENT  0xFFFF

/** Input parameters when configuring a PWM channel mode */
typedef struct {
	uint32_t pwm_period_ticks;
	bool center_aligned;
	bool active_high_out;
	bool enable_period_start_event;
	bool deadtime_enabled;
	uint16_t deadtime_pwmh_ticks;
	uint16_t deadtime_pwml_ticks;
} pwm_opt_t;

uint16_t pwm_calc_deadtime_ticks(uint32_t deadtime_ns);
uint32_t pwm_calc_period_ticks(uint32_t pwm_frequency, bool center_aligned);

uint32_t pwm_init(Pwm *p_pwm, pwm_opt_t *settings, uint8_t num_channels);
bool pwm_is_next_period(Pwm *p_pwm);
void pwm_update_channel_data(Pwm *p_pwm, pwm_opt_t *p_channel, uint8_t channel, uint32_t duty);
void pwm_channel_enable(Pwm *p_pwm, uint8_t channel);
void pwm_channel_disable(Pwm *p_pwm, uint8_t channel);

void pwm_enable_override(Pwm *p_pwm, uint8_t channel);
void pwm_disable_override(Pwm *p_pwm, uint8_t channel);

#endif /* PWM_H_INCLUDED */
