/*
 * usart_rs232.c
 *
 * Created: 14.02.2016 17.06.40
 *  Author: Simen
 */ 

#include <stdlib.h>
#include "usart_rs232.h"
#include "../../pmc.h"
#include "../../pio.h"
#include "../Common_driver/usart.h"

void usart_rs232_init(Usart *p_usart, usart_rs232_opt_t *options){
	//Enable peripheral clock
	if(p_usart == USART0){
		pmc_enable_periph_clk(ID_USART0);
		pio_setMux(PIOB,  0, C); //RX
		pio_setMux(PIOB,  1, C); //TX
		if(options->enable_hw_flowcontrol){
			pio_setMux(PIOB,  2, C); //CTS
			pio_setMux(PIOB,  3, C); //RTS
		}
	}
	if(p_usart == USART1){
		pmc_enable_periph_clk(ID_USART1);
		//Set output pin muxes
		pio_setMux(PIOA, 21, A); //RX
		pio_setMux(PIOB,  4, D); //TX
		if(options->enable_hw_flowcontrol){
			pio_setMux(PIOA, 25, A); //CTS
			pio_setMux(PIOA, 24, A); //RTS
		}
	}
	if(p_usart == USART2){
		pmc_enable_periph_clk(ID_USART2);
		pio_setMux(PIOD, 15, B); //RX
		pio_setMux(PIOD, 16, B); //TX
		if(options->enable_hw_flowcontrol){
			pio_setMux(PIOD, 19, B); //CTS
			pio_setMux(PIOD, 18, B); //RTS
		}
	}

	static uint32_t ul_reg_val;

	/* Reset the USART and shut down TX and RX. */
	usart_reset(p_usart);

	ul_reg_val = 0;
	/* Check whether the input values are legal. */
	if (!options || usart_set_async_baudrate(p_usart, options->baudrate, pmc_getPeripheralClockValue())) {
		return ;
	}

	/* Configure the USART option. */
	ul_reg_val |= options->char_length | options->parity_type |options->stop_bits;

	if(options->enable_hw_flowcontrol){
		ul_reg_val |= US_MR_USART_MODE_HW_HANDSHAKING;
	}
	else{
		/* Configure the USART mode as normal mode. */
		ul_reg_val |= US_MR_USART_MODE_NORMAL;
	}

	p_usart->US_MR |= ul_reg_val;



	usart_enable_rx(p_usart);
	usart_enable_tx(p_usart);
}

void usart_rs232_write_string(Usart *p_usart, const unsigned char *data, size_t len){
	for(uint16_t i = 0; (i < len) && (data[i] != '\0'); i++){
		usart_putchar(p_usart, data[i]);
	}
	
}

void usart_rs232_read_packet(Usart *p_usart, unsigned char* data, size_t len){
	 do{
		usart_getchar(p_usart, data);
		len--;
		data++;
	}while (len && *(data - 1)); //Check for end of string, and length limit
}

uint32_t usart_rs232_is_tx_empty(Usart *p_usart){
	return usart_is_tx_empty(p_usart);
}

uint32_t usart_rs232_is_rx_ready(Usart *p_usart){
	return usart_is_rx_ready(p_usart);
}

uint32_t usart_rs232_is_tx_ready(Usart *p_usart){
	return usart_is_tx_ready(p_usart);
}