#ifndef PMC_H_INCLUDED
#define PMC_H_INCLUDED

#include <sam.h>

typedef enum{
	PMC_PCK_0 = 0,
	PMC_PCK_1 = 1,
	PMC_PCK_2 = 2,
	PMC_PCK_5 = 5,
	PMC_PCK_6 = 6
} ProgrammableClockSource;


typedef enum {
	EXT_16MHZ,
	EXT_12MHZ,
	INT_4MHZ
} MainClockFrequency;


typedef enum {CLK_1 = PMC_MCKR_PRES_CLK_1,
	CLK_2 = PMC_MCKR_PRES_CLK_2,
	CLK_4 = PMC_MCKR_PRES_CLK_4,
	CLK_8 = PMC_MCKR_PRES_CLK_8,
	CLK_16 = PMC_MCKR_PRES_CLK_16,
	CLK_32 = PMC_MCKR_PRES_CLK_32,
	CLK_64 = PMC_MCKR_PRES_CLK_64,
	CLK_3 = PMC_MCKR_PRES_CLK_3
} ProcessorClockPrescaler;

typedef enum {
	PLLA_CLOCK = PMC_MCKR_CSS_PLLA_CLK
} MasterClockSource;

struct PmcInit {
	MainClockFrequency main_freq;
	MasterClockSource css;
	ProcessorClockPrescaler pres;
	uint8_t divide;
	uint8_t multiply;
};




void pmc_init_clocks(struct PmcInit pmc_init_struct);

uint32_t pmc_calcCoreClock(struct PmcInit pmc_init_struct);
extern uint32_t pmc_getCoreClockValue(void);
extern uint32_t pmc_getPeripheralClockValue(void);

//Enable/disable module clocks
uint32_t pmc_enable_periph_clk(uint32_t irqnNumber);
uint32_t pmc_disable_periph_clk(uint32_t irqnNumber);

//Setup programmable clocks
void pmc_enable_programmable_clk(ProgrammableClockSource irqnNumber);
void pmc_disable_programmable_clk(ProgrammableClockSource irqnNumber);
uint32_t pmc_switch_programmable_clk_to_mainck(ProgrammableClockSource irqnNumber, uint32_t pres);
uint32_t pmc_switch_programmable_clk_to_pllack(ProgrammableClockSource irqnNumber, uint32_t pres);
uint32_t pmc_switch_programmable_clk_to_upllck(ProgrammableClockSource irqnNumber, uint32_t pres);
uint32_t pmc_is_programmable_clk_enabled(ProgrammableClockSource irqnNumber);

//UPLL interface, used by CAN and USB
void pmc_enable_udpck(void);
void pmc_disable_udpck(void);
void pmc_switch_udpck_to_upllck(uint32_t usbdiv);


#endif /* PMC_H_INCLUDED */
