#ifndef SPI_H_
#define SPI_H_

#include <sam.h>
#include <stdbool.h>

/** Time-out value (number of attempts). */
#define SPI_TIMEOUT       15000

/** Status codes used by the SPI driver. */
typedef enum
{
	SPI_ERROR = -1,
	SPI_OK = 0,
	SPI_ERROR_TIMEOUT = 1,
	SPI_ERROR_ARGUMENT,
	SPI_ERROR_OVERRUN,
	SPI_ERROR_MODE_FAULT,
	SPI_ERROR_OVERRUN_AND_MODE_FAULT
} spi_status_t;

enum NCPS0pin{none0, PB2};
enum NCPS1pin{none1, PA31,	PD25};
enum NCPS2pin{none2, PD12};
enum NCPS3pin{none3, PD27};
	
enum SpiMode{
	MODE_0 = 0x02, // CPOL 0 , NCPHA 1
	MODE_1 = 0x00, // CPOL 0 , NCPHA 0
	MODE_2 = 0x03, // CPOL 1 , NCPHA 1
	MODE_3 = 0x01 // CPOL 1 , NCPHA 0
};

enum SpiChipSelect{
	NPCS0 = 0,
	NPCS1,
	NPCS2,
	NPCS3,
};

struct SpiMasterSettings {
	enum NCPS0pin cs_0;
	enum NCPS1pin cs_1;
	enum NCPS2pin cs_2;
	enum NCPS3pin cs_3;
	uint16_t CSDelay_ns;
};

enum SpiDataWidth{
	SPI_8BIT_DATA  = (0 << 4),
	SPI_9BIT_DATA  = (1 << 4),
	SPI_10BIT_DATA = (2 << 4),
	SPI_11BIT_DATA = (3 << 4),
	SPI_12BIT_DATA = (4 << 4),
	SPI_13BIT_DATA = (5 << 4),
	SPI_14BIT_DATA = (6 << 4),
	SPI_15BIT_DATA = (7 << 4),
	SPI_16BIT_DATA = (8 << 4),
};

// This struct is for setting correct settings for each 
// chip select in use. Each chip select can use the SPI
// bus differently.
struct SpiSlaveSettings {
	enum SpiMode spi_mode;
	enum SpiChipSelect chip_select;
	bool raise_CS_every_transfer;
	uint32_t spi_baudRate_Hz;
	enum SpiDataWidth data_width;
};
	
spi_status_t spi_transceive(uint16_t *tx_buffer,  uint16_t *rx_buffer, uint32_t buffer_size, enum SpiChipSelect chip_select);

void spi_master_init(Spi *spi, struct SpiMasterSettings spi_settings);
void spi_chip_select_init(Spi *spi, struct SpiSlaveSettings spi_cs_settings);

uint32_t spi_generateControlWord(bool last_xfer,enum SpiChipSelect chip_select, uint16_t data);
uint32_t spi_flush_rx_register(Spi *spi);

const volatile uint32_t *spi_get_rx_register_address(Spi *spi);
volatile uint32_t *spi_get_tx_register_address(Spi *spi);

#endif /* SPI_H_ */