﻿/*
 * uart.h
 *
 * Created: 02.12.2015 18:05:45
 *  Author: Ørjan
 */ 


#ifndef UART_H_
#define UART_H_
#include <sam.h>
#include <stdbool.h>


enum uart_transmitStatus {
	UART_TRANSMIT_SUCCESS,
	UART_TRANSMIT_FAILURE
	};
	
enum uart_receiveStatus {
	UART_RECEIVE_SUCCESS,
	UART_RECEIVE_FAILURE
	};

enum uart_parity {
	EVEN_PARITY  = 0,
	ODD_PARITY   = 1,
	SPACE_PARITY = 2,
	MARK_PARITY  = 3,
	NO_PARITY    = 4
	};

enum uart_mode {
	NORMAL_MODE			 = 0,
	AUTOMATIC_MODE		 = 1,
	LOCAL_LOOPBACK_MODE  = 2,
	REMOTE_LOOPBACK_MODE = 3
	};
	
enum uart_TX_pin {
	UART0_TX_PIN_PA10,
	UART1_TX_PIN_PA4,
	UART1_TX_PIN_PA6,
	UART1_TX_PIN_PD26,
	UART2_TX_PIN_PD26,
	UART3_TX_PIN_PD30,
	UART3_TX_PIN_PD31,
	UART4_TX_PIN_PD3,
	UART4_TX_PIN_PD19
	};

enum uart_RX_pin {
	UART0_RX_PIN_PA9,
	UART1_RX_PIN_PA5,
	UART2_RX_PIN_PD25,
	UART3_RX_PIN_PD28,
	UART4_RX_PIN_PD18
	};

struct uart_config {
	Uart * uartID;
	enum uart_TX_pin TX_pin;
	enum uart_RX_pin RX_pin;
	enum uart_parity parity;
	enum uart_mode mode;
	uint32_t baudrate;
	};

void uart_init			(struct uart_config config);
void uart_transmitData (Uart * uartID, uint8_t data);
enum uart_receiveStatus  uart_receiveData	(Uart * uartID, uint8_t * data);
void uart_transmitBuffer(Uart *uartID, uint8_t *tx_buffer, uint32_t buffer_size);


void uart_resetStatusRegister(Uart *uartID);
void uart_setCompareRegisterFlagOnly(Uart *uartID, uint8_t cmp_value_1, uint8_t cmp_value_2);
void uart_enableCompareInterrupt(Uart *uartID);
void uart_enableRxReadyInterrupt(Uart *uartID);

#endif /* UART_H_ */