
#include <sam.h>
#include "pmc.h"
#include "stdlib.h"
#include "xdmac_wrapper.h"
#include "xdmac.h"


uint32_t xdmac_get_type( Xdmac *xdmac);
uint32_t xdmac_get_config(Xdmac *xdmac);
uint32_t xdmac_get_arbiter(Xdmac *xdmac);
uint32_t xdmac_get_interrupt_mask(Xdmac *xdmac);
uint32_t xdmac_get_interrupt_status(Xdmac *xdmac);
uint32_t xdmac_channel_get_status(Xdmac *xdmac);
void xdmac_channel_read_suspend(Xdmac *xdmac, uint32_t channel_num);
void xdmac_channel_write_suspend(Xdmac *xdmac, uint32_t channel_num);
void xdmac_channel_readwrite_suspend(Xdmac *xdmac, uint32_t channel_num);
void xdmac_channel_readwrite_resume(Xdmac *xdmac, uint32_t channel_num);
uint32_t xdmac_get_software_request_status(Xdmac *xdmac);
void xdmac_channel_software_flush_request(Xdmac *xdmac, uint32_t channel_num);
void xdmac_channel_set_source_addr(Xdmac *xdmac, uint32_t channel_num, volatile uint32_t *src_addr);
void xdmac_channel_set_block_control(Xdmac *xdmac, uint32_t channel_num, uint32_t blen);
void xdmac_channel_set_config(Xdmac *xdmac, uint32_t channel_num, uint32_t config);
void xdmac_channel_set_datastride_mempattern(Xdmac *xdmac, uint32_t channel_num, uint32_t dds_msp);
void xdmac_channel_set_microblock_control(Xdmac *xdmac, uint32_t channel_num, uint32_t ublen);
void xdmac_channel_set_source_microblock_stride(Xdmac *xdmac, uint32_t channel_num, uint32_t subs);
void xdmac_channel_set_destination_microblock_stride(Xdmac *xdmac, uint32_t channel_num, uint32_t dubs);
void xdmac_channel_software_request(Xdmac *xdmac, uint32_t channel_num);

void (*callback_functions[24])(uint32_t interrupt_status) = {NULL};

void xdma_setup_memory_transfer(Xdmac *xdmac, Xdma_transfer_opts_t* opts){
	
	pmc_enable_periph_clk(ID_XDMAC);		
			
			
	volatile struct XdmacChannelConfig cfg;
	cfg.destination_address = opts->destination_adress;
	cfg.source_address = opts->source_adress;
		
	cfg.microblock_length = opts->array_size;
		
	cfg.block_length = opts->num_transfers -1;
		
	cfg.mempattern_datastride			  = 0;
	cfg.source_microblock_datastride	  = 0;
	cfg.destination_microblock_datastride = 0;
		
	cfg.config_register =	(XDMAC_CC_TYPE_MEM_TRAN			|
							opts->element_size);
							
	cfg.config_register |= (opts->source_bus == XDMAC_BUS_0) ? XDMAC_CC_SIF_AHB_IF0 : XDMAC_CC_SIF_AHB_IF1;
	cfg.config_register |= (opts->destination_bus == XDMAC_BUS_0) ? XDMAC_CC_DIF_AHB_IF0 : XDMAC_CC_DIF_AHB_IF1;
		
	if(opts->array_size > 16){
		cfg.config_register |= XDMAC_CC_MBSIZE_SIXTEEN;
	}
	else if(opts->array_size > 8){
		cfg.config_register |= XDMAC_CC_MBSIZE_EIGHT;
	}
	else if(opts->array_size > 4){
		cfg.config_register |= XDMAC_CC_MBSIZE_FOUR;
	}
	else{
		cfg.config_register |= XDMAC_CC_MBSIZE_SINGLE;
	}
		
	if(opts->trigger == XDMAC_EVENT_SOFTWARE){
		cfg.config_register |= XDMAC_CC_SWREQ;
	}
	else{
		cfg.config_register |= XDMAC_CC_PERID(opts->trigger);
	}
		
		
	xdmac_configure_transfer(xdmac, opts->channel_number, &cfg);
			
	if(opts->interrupt_mask != 0){
		callback_functions[opts->channel_number] = opts->callback;
		xdmac_enable_interrupt(xdmac, opts->channel_number);
		xdmac_channel_enable_interrupt(xdmac, opts->channel_number, opts->interrupt_mask);
		NVIC_ClearPendingIRQ(XDMAC_IRQn);
		NVIC_SetPriority( XDMAC_IRQn, opts->interrupt_priority);
		NVIC_EnableIRQ(XDMAC_IRQn);
	}
}

void xdma_setup_peripheral_transfer(Xdmac *xdmac, Xdma_transfer_opts_t* opts){	
	
	pmc_enable_periph_clk(ID_XDMAC);	
		
	volatile struct XdmacChannelConfig cfg;
	
	cfg.source_address = opts->source_adress;
	cfg.destination_address = opts->destination_adress;
	
	cfg.microblock_length = opts->array_size;
	
	cfg.block_length = opts->num_transfers;
	
	cfg.mempattern_datastride				= 0;
	cfg.source_microblock_datastride		= 0;
	cfg.destination_microblock_datastride	= 0;
	
	cfg.config_register =	(XDMAC_CC_TYPE_PER_TRAN			|
	XDMAC_CC_MBSIZE_SINGLE			|
	opts->peripheral_transfer_dir	|
	XDMAC_CC_CSIZE_CHK_1			|
	opts->element_size);
	
	cfg.config_register |= (opts->source_bus == XDMAC_BUS_0) ? XDMAC_CC_SIF_AHB_IF0 : XDMAC_CC_SIF_AHB_IF1;
	cfg.config_register |= (opts->destination_bus == XDMAC_BUS_0) ? XDMAC_CC_DIF_AHB_IF0 : XDMAC_CC_DIF_AHB_IF1;
							
	if(opts->trigger == XDMAC_EVENT_SOFTWARE){
		cfg.config_register |= XDMAC_CC_SWREQ;
	}
	else{
		cfg.config_register |= XDMAC_CC_PERID(opts->trigger);
	}
		
	if(opts->peripheral_transfer_dir == XDMAC_DIR_MEM2PER){
		cfg.config_register |=	XDMAC_CC_SAM_INCREMENTED_AM |
								XDMAC_CC_DAM_FIXED_AM;
	}
	else{
		cfg.config_register |=	XDMAC_CC_DAM_INCREMENTED_AM |
								XDMAC_CC_SAM_FIXED_AM;
	}
		
	xdmac_configure_transfer(xdmac, opts->channel_number, &cfg);
	
	if(opts->interrupt_mask != 0){
		callback_functions[opts->channel_number] = opts->callback;
		xdmac_enable_interrupt(xdmac, opts->channel_number);
		xdmac_channel_enable_interrupt(xdmac, opts->channel_number, opts->interrupt_mask);
		NVIC_ClearPendingIRQ(XDMAC_IRQn);
		NVIC_SetPriority( XDMAC_IRQn, opts->interrupt_priority);
		NVIC_EnableIRQ(XDMAC_IRQn);
	}
}


void xdma_start_transfer(Xdmac *xdmac, Xdma_transfer_opts_t* opts){
	xdmac_channel_get_interrupt_status( xdmac, opts->channel_number);
	
	xdmac_channel_set_destination_addr(xdmac, opts->channel_number, opts->destination_adress);
	xdmac_channel_set_source_addr(xdmac, opts->channel_number, opts->source_adress);
		
	xdmac_channel_enable(xdmac, opts->channel_number);

}


void xdma_stop_transfer(Xdmac *xdmac, Xdma_transfer_opts_t* opts){
	xdmac_channel_disable(xdmac, opts->channel_number);
}

void XDMAC_Handler(void){
	uint32_t interrupt_status = 0;
	
	for(uint8_t i = 0; i < 24; i++){
		interrupt_status = xdmac_channel_get_interrupt_status(XDMAC, i);
		if(interrupt_status && callback_functions[i] != NULL){
			callback_functions[i](interrupt_status);
		}
	}
}

/**
 * \brief Configure DMA for a transfer.
 *
 * \param[out] xdmac Module hardware register base address pointer.
 * \param[in] channel_num The used channel number.
 * \param[in] cfg   The configuration for used channel
 */
void xdmac_configure_transfer(Xdmac *xdmac,
		uint32_t channel_num, volatile struct XdmacChannelConfig *cfg)
{
	
	xdmac_channel_get_interrupt_status( xdmac, channel_num);
	xdmac_channel_set_source_addr(xdmac, channel_num, cfg->source_address);
	xdmac_channel_set_destination_addr(xdmac, channel_num, cfg->destination_address);
	xdmac_channel_set_microblock_control(xdmac, channel_num, cfg->microblock_length);
	xdmac_channel_set_block_control(xdmac, channel_num, cfg->block_length);
	xdmac_channel_set_datastride_mempattern(xdmac, channel_num, cfg->mempattern_datastride);
	xdmac_channel_set_source_microblock_stride(xdmac, channel_num, cfg->source_microblock_datastride);
	xdmac_channel_set_destination_microblock_stride(xdmac, channel_num, cfg->destination_microblock_datastride);
	xdmac_channel_set_config(xdmac, channel_num, cfg->config_register );
}

/**desrg
 * \brief Get XDMAC global type.
 *
 * \param[out] xdmac Module hardware register base address pointer.
 */
 uint32_t xdmac_get_type( Xdmac *xdmac)
{
	return xdmac->XDMAC_GTYPE;
}

/**
 * \brief Get XDMAC global configuration.
 *
 * \param[out] xdmac Module hardware register base address pointer.
 */
 uint32_t xdmac_get_config(Xdmac *xdmac)
{
	
	return xdmac->XDMAC_GCFG;
}

/**
 * \brief Get XDMAC global weighted arbiter configuration.
 *
 * \param[out] xdmac Module hardware register base address pointer.
 */
 uint32_t xdmac_get_arbiter(Xdmac *xdmac)
{
	
	return xdmac->XDMAC_GWAC;
}

/**
 * \brief Enables XDMAC global interrupt.
 *
 * \param[out] xdmac Module hardware register base address pointer.
 * \param[in] mask A bitmask of channels to be enabled interrupt.
 */
 void xdmac_enable_interrupt(Xdmac *xdmac, uint32_t mask)
{
	
	xdmac->XDMAC_GIE = ( XDMAC_GIE_IE0 << mask) ;
}

/**
 * \brief Disables XDMAC global interrupt
 *
 * \param[out] xdmac Module hardware register base address pointer.
 * \param[in] mask A bitmask of channels to be disabled interrupt.
 */
 void xdmac_disable_interrupt(Xdmac *xdmac, uint32_t mask)
{
	
	xdmac->XDMAC_GID = (XDMAC_GID_ID0 << mask);
}

/**
 * \brief Get XDMAC global interrupt mask.
 *
 * \param[out] xdmac Module hardware register base address pointer.
 */
 uint32_t xdmac_get_interrupt_mask(Xdmac *xdmac)
{
	
	return (xdmac->XDMAC_GIM);
}

/**
 * \brief Get XDMAC global interrupt status.
 *
 * \param[out] xdmac Module hardware register base address pointer.
 */
 uint32_t xdmac_get_interrupt_status(Xdmac *xdmac)
{
	
	return (xdmac->XDMAC_GIS);
}

/**
 * \brief enables the relevant channel of given XDMAC.
 *
 * \param[out] xdmac Module hardware register base address pointer.
 * \param[in]  channel_num  XDMA Channel number (range 0 to 23)
 */
 void xdmac_channel_enable(Xdmac *xdmac, uint32_t channel_num)
{
	
	xdmac->XDMAC_GE = (XDMAC_GE_EN0 << channel_num);
}

/**
 * \brief Disables the relevant channel of given XDMAC.
 *
 * \param[out] xdmac Module hardware register base address pointer.
 * \param[in]  channel_num  XDMA Channel number (range 0 to 23)
 */
 void xdmac_channel_disable(Xdmac *xdmac, uint32_t channel_num)
{
	
	xdmac->XDMAC_GD =(XDMAC_GD_DI0 << channel_num);
}

/**
 * \brief Get Global channel status of given XDMAC.
 * \note: When set to 1, this bit indicates that the channel x is enabled.
          If a channel disable request is issued, this bit remains asserted
          until pending transaction is completed.
 * \param[out] xdmac Module hardware register base address pointer.
 */
 uint32_t xdmac_channel_get_status(Xdmac *xdmac)
{
	
	return xdmac->XDMAC_GS;
}

/**
 * \brief Suspend the relevant channel's read.
 *
 * \param[out] xdmac Module hardware register base address pointer.
 * \param[in] channel_num  XDMA Channel number (range 0 to 23).
 */
 void xdmac_channel_read_suspend(Xdmac *xdmac, uint32_t channel_num)
{
	xdmac->XDMAC_GRS |= XDMAC_GRS_RS0 << channel_num;
}

/**
 * \brief Suspend the relevant channel's write.
 *
 * \param[out] xdmac Module hardware register base address pointer.
 * \param[in] channel_num  XDMA Channel number (range 0 to 23).
 */
 void xdmac_channel_write_suspend(Xdmac *xdmac, uint32_t channel_num)
{
	
	xdmac->XDMAC_GWS |= XDMAC_GWS_WS0 << channel_num;
}

/**
 * \brief Suspend the relevant channel's read & write.
 *
 * \param[out] xdmac Module hardware register base address pointer.
 * \param[in] channel_num  XDMA Channel number (range 0 to 23).
 */
 void xdmac_channel_readwrite_suspend(Xdmac *xdmac, uint32_t channel_num)
{
	
	xdmac->XDMAC_GRWS = (XDMAC_GRWS_RWS0 << channel_num);
}

/**
 * \brief Resume the relevant channel's read & write.
 *
 * \param[out] xdmac Module hardware register base address pointer.
 * \param[in] channel_num  XDMA Channel number (range 0 to 23).
 */
 void xdmac_channel_readwrite_resume(Xdmac *xdmac, uint32_t channel_num)
{
	xdmac->XDMAC_GRWR = (XDMAC_GRWR_RWR0 << channel_num);
}

/**
 * \brief Set software transfer request on the relevant channel.
 *
 * \param[out] xdmac Module hardware register base address pointer.
 * \param[in] channel_num  XDMA Channel number (range 0 to 23).
 */
 void xdmac_channel_software_request(Xdmac *xdmac, uint32_t channel_num)
{

	xdmac->XDMAC_GSWR = (XDMAC_GSWR_SWREQ0 << channel_num);
}

/**
 * \brief Get software transfer status of the relevant channel.
 *
 * \param[out] xdmac Module hardware register base address pointer.
 */
 uint32_t xdmac_get_software_request_status(Xdmac *xdmac)
{
	
	return xdmac->XDMAC_GSWS;
}

/**
 * \brief Enable interrupt with mask on the relevant channel of given XDMA.
 *
 * \param[out] xdmac Module hardware register base address pointer.
 * \param[in] channel_num  XDMA Channel number (range 0 to 23).
 * \param[in] mask Interrupt mask.
 */
 void xdmac_channel_enable_interrupt(Xdmac *xdmac, uint32_t channel_num, uint32_t mask)
{
	
	xdmac->XDMAC_CHID[channel_num].XDMAC_CIE = mask;
}

/**
 * \brief Disable interrupt with mask on the relevant channel of given XDMA.
 *
 * \param[out] xdmac Module hardware register base address pointer.
 * \param[in] channel_num  XDMA Channel number (range 0 to 23).
 * \param[in] mask Interrupt mask.
 */
 void xdmac_channel_disable_interrupt(Xdmac *xdmac, uint32_t channel_num, uint32_t mask)
{
	
	xdmac->XDMAC_CHID[channel_num].XDMAC_CID = mask;
}

/**
 * \brief Get interrupt mask for the relevant channel of given XDMA.
 *
 * \param[out] xdmac Module hardware register base address pointer.
 * \param[in] channel_num  XDMA Channel number (range 0 to 23).
 */
 uint32_t xdmac_channel_get_interrupt_mask(Xdmac *xdmac, uint32_t channel_num)
{
	return xdmac->XDMAC_CHID[channel_num].XDMAC_CIM;
}

/**
 * \brief Get interrupt status for the relevant channel of given XDMA.
 *
 * \param[out] xdmac Module hardware register base address pointer.
 * \param[in] channel_num  XDMA Channel number (range 0 to 23).
 */
 uint32_t xdmac_channel_get_interrupt_status(Xdmac *xdmac, uint32_t channel_num)
{

	return xdmac->XDMAC_CHID[channel_num].XDMAC_CIS;
}

/**
 * \brief Set software flush request on the relevant channel.
 *
 * \param[out] xdmac Module hardware register base address pointer.
 * \param[in] channel_num  XDMA Channel number (range 0 to 23).
 */
 void xdmac_channel_software_flush_request(Xdmac *xdmac, uint32_t channel_num)
{

	xdmac->XDMAC_GSWF = (XDMAC_GSWF_SWF0 << channel_num);
	//while( !(xdmac_channel_get_interrupt_status(xdmac, channel_num) & XDMAC_CIS_FIS) );
}

/**
 * \brief Set source address for the relevant channel of given XDMA.
 *
 * \param[out] xdmac Module hardware register base address pointer
 * \param[in] channel_num  DMA Channel number (range 0 to 23)
 * \param[in] src_addr Source address
 */
 void xdmac_channel_set_source_addr(Xdmac *xdmac, uint32_t channel_num, volatile uint32_t *src_addr)
{

	xdmac->XDMAC_CHID[channel_num].XDMAC_CSA = (uint32_t)src_addr;
}

/**
 * \brief Set destination address for the relevant channel of given XDMA.
 *
 * \param[out] xdmac Module hardware register base address pointer
 * \param[in] channel_num  DMA Channel number (range 0 to 23)
 * \param[in] dst_addr Destination address
 */
 void xdmac_channel_set_destination_addr(Xdmac *xdmac, uint32_t channel_num, volatile uint32_t *dst_addr)
{

	xdmac->XDMAC_CHID[channel_num].XDMAC_CDA = (uint32_t)dst_addr;
}

/**
 * \brief Set next descriptor's address & interface for the relevant channel of given XDMA.
 *
 * \param[out] xdmac Module hardware register base address pointer
 * \param[in] channel_num  DMA Channel number (range 0 to 23)
 * \param[in] desc_addr Address of next descriptor.
 * \param[in] ndaif Interface of next descriptor.
 */
 void xdmac_channel_set_descriptor_addr(Xdmac *xdmac, uint32_t channel_num, volatile uint32_t *desc_addr, uint8_t ndaif)
{

	xdmac->XDMAC_CHID[channel_num].XDMAC_CNDA = (((uint32_t)desc_addr) & 0xFFFFFFFC) | ndaif;
}

/**
 * \brief Set next descriptor's configuration for the relevant channel of given XDMA.
 *
 * \param[out] xdmac Module hardware register base address pointer
 * \param[in] channel_num  DMA Channel number (range 0 to 23)
 * \param[in] config Configuration of next descriptor.
 */
 void xdmac_channel_set_descriptor_control(Xdmac *xdmac, uint32_t channel_num, uint32_t config)
{

	xdmac->XDMAC_CHID[channel_num].XDMAC_CNDC = config;
}

/**
 * \brief Set microblock length for the relevant channel of given XDMA.
 *
 * \param[out] xdmac Module hardware register base address pointer
 * \param[in] channel_num  DMA Channel number (range 0 to 23)
 * \param[in] ublen Microblock length.
 */
 void xdmac_channel_set_microblock_control(Xdmac *xdmac, uint32_t channel_num, uint32_t ublen)
{

	xdmac->XDMAC_CHID[channel_num].XDMAC_CUBC = XDMAC_CUBC_UBLEN(ublen);
}

/**
 * \brief Set block length for the relevant channel of given XDMA.
 *
 * \param[out] xdmac Module hardware register base address pointer
 * \param[in] channel_num  XDMA Channel number (range 0 to 23)
 * \param[in] blen Block length.
 */
 void xdmac_channel_set_block_control(Xdmac *xdmac, uint32_t channel_num, uint32_t blen)
{

	xdmac->XDMAC_CHID[channel_num].XDMAC_CBC = XDMAC_CBC_BLEN(blen);
}

/**
 * \brief Set configuration for the relevant channel of given XDMA.
 *
 * \param[out] xdmac Module hardware register base address pointer
 * \param[in] channel_num  XDMA Channel number (range 0 to 23)
 * \param[in] config Channel configuration.
 */
 void xdmac_channel_set_config(Xdmac *xdmac, uint32_t channel_num, uint32_t config)
{

	xdmac->XDMAC_CHID[channel_num].XDMAC_CC = config;
}

/**
 * \brief Set the relevant channel's data stride memory pattern of given XDMA.
 *
 * \param[out] xdmac Module hardware register base address pointer
 * \param[in] channel_num  XDMA Channel number (range 0 to 23)
 * \param[in] dds_msp Data stride memory pattern.
 */
 void xdmac_channel_set_datastride_mempattern(Xdmac *xdmac, uint32_t channel_num, uint32_t dds_msp)
{
	xdmac->XDMAC_CHID[channel_num].XDMAC_CDS_MSP = dds_msp;
}

/**
 * \brief Set the relevant channel's source microblock stride of given XDMA.
 *
 * \param[out] xdmac Module hardware register base address pointer
 * \param[in] channel_num  XDMA Channel number (range 0 to 23)
 * \param[in] subs Source microblock stride.
 */
 void xdmac_channel_set_source_microblock_stride(Xdmac *xdmac,
		uint32_t channel_num, uint32_t subs)
{
	xdmac->XDMAC_CHID[channel_num].XDMAC_CSUS = XDMAC_CSUS_SUBS(subs);
}

/**
 * \brief Set the relevant channel's destination microblock stride of given XDMA.
 *
 * \param[out] xdmac Module hardware register base address pointer
 * \param[in] channel_num  XDMA Channel number (range 0 to 23)
 * \param[in] dubs Destination microblock stride.
 */
 void xdmac_channel_set_destination_microblock_stride(Xdmac *xdmac,
		uint32_t channel_num, uint32_t dubs)
{
	xdmac->XDMAC_CHID[channel_num].XDMAC_CDUS = XDMAC_CDUS_DUBS(dubs);
}
