#ifndef XDMAC_WRAPPER_H_INCLUDED
#define XDMAC_WRAPPER_H_INCLUDED


#include <sam.h>

typedef enum{
/** DMA channel hardware interface number */
	XDMAC_EVENT_HWID_HSMCI			= 0 ,
	XDMAC_EVENT_HWID_SPI0_TX		= 1 ,
	XDMAC_EVENT_HWID_SPI0_RX		= 2 ,
	XDMAC_EVENT_HWID_SPI1_TX		= 3 ,
	XDMAC_EVENT_HWID_SPI1_RX		= 4 ,
	XDMAC_EVENT_HWID_QSPI_TX		= 5 ,
	XDMAC_EVENT_HWID_QSPI_RX		= 6 ,
	XDMAC_EVENT_HWID_USART0_TX		= 7 ,
	XDMAC_EVENT_HWID_USART0_RX		= 8 ,
	XDMAC_EVENT_HWID_USART1_TX		= 9 ,
	XDMAC_EVENT_HWID_USART1_RX		= 10,
	XDMAC_EVENT_HWID_USART2_TX		= 11,
	XDMAC_EVENT_HWID_USART2_RX		= 12,
	XDMAC_EVENT_HWID_PWM0			= 13,
	XDMAC_EVENT_HWID_TWIHS0_TX		= 14,
	XDMAC_EVENT_HWID_TWIHS0_RX		= 15,
	XDMAC_EVENT_HWID_TWIHS1_TX		= 16,
	XDMAC_EVENT_HWID_TWIHS1_RX		= 17,
	XDMAC_EVENT_HWID_TWIHS2_TX		= 18,
	XDMAC_EVENT_HWID_TWIHS2_RX		= 19,
	XDMAC_EVENT_HWID_UART0_TX		= 20,
	XDMAC_EVENT_HWID_UART0_RX		= 21,
	XDMAC_EVENT_HWID_UART1_TX		= 22,
	XDMAC_EVENT_HWID_UART1_RX		= 23,
	XDMAC_EVENT_HWID_UART2_TX		= 24,
	XDMAC_EVENT_HWID_UART2_RX		= 25,
	XDMAC_EVENT_HWID_UART3_TX		= 26,
	XDMAC_EVENT_HWID_UART3_RX		= 27,
	XDMAC_EVENT_HWID_UART4_TX		= 28,
	XDMAC_EVENT_HWID_UART4_RX		= 29,
	XDMAC_EVENT_HWID_DAC			= 30,
	XDMAC_EVENT_HWID_SSC_TX			= 32,
	XDMAC_EVENT_HWID_SSC_RX			= 33,
	XDMAC_EVENT_HWID_PIOA			= 34,
	XDMAC_EVENT_HWID_AFEC0			= 35,
	XDMAC_EVENT_HWID_AFEC1			= 36,
	XDMAC_EVENT_HWID_AES_TX			= 37,
	XDMAC_EVENT_HWID_AES_RX			= 38,
	XDMAC_EVENT_HWID_PWM1			= 39,
	XDMAC_EVENT_HWID_TC0			= 40,
	XDMAC_EVENT_HWID_TC1			= 41,
	XDMAC_EVENT_HWID_TC2			= 42,
	XDMAC_EVENT_HWID_TC3			= 43,
	XDMAC_EVENT_SOFTWARE			= 0xFF
}Xdma_event_num_t;

typedef enum{
	XDMAC_DIR_PER2MEM = XDMAC_CC_DSYNC_PER2MEM,
	XDMAC_DIR_MEM2PER = XDMAC_CC_DSYNC_MEM2PER
}Xdmac_ch_dir_t;

typedef enum{
	XDMAC_SIZE_8			= XDMAC_CC_DWIDTH_BYTE,
	XDMAC_SIZE_16			= XDMAC_CC_DWIDTH_HALFWORD,
	XDMAC_SIZE_32			= XDMAC_CC_DWIDTH_WORD
}Xdmac_ch_Sz_t;

typedef enum{
	XDMAC_BUS_0,
	XDMAC_BUS_1
}Xdmac_bus;

typedef struct{
	uint32_t			channel_number;
	Xdmac_ch_Sz_t		element_size;
	uint32_t			array_size;
	uint32_t			num_transfers;
	Xdmac_ch_dir_t		peripheral_transfer_dir;
	volatile uint32_t*	volatile source_adress;
	volatile uint32_t*	volatile destination_adress;
	Xdma_event_num_t	trigger;
	uint32_t			interrupt_mask;
	uint32_t			interrupt_priority;
	void				(*callback)(uint32_t interrupt_status);
	Xdmac_bus	source_bus;
	Xdmac_bus	destination_bus;
}Xdma_transfer_opts_t;


void xdma_setup_peripheral_transfer(Xdmac *xdmac, Xdma_transfer_opts_t* opts);
void xdma_setup_memory_transfer(Xdmac *xdmac, Xdma_transfer_opts_t* opts);
void xdma_start_transfer(Xdmac *xdmac, Xdma_transfer_opts_t* opts);
void xdma_stop_transfer(Xdmac *xdmac, Xdma_transfer_opts_t* opts);

#endif /* XDMAC_H_INCLUDED */
