/*
 * AngleConversions.c
 *
 * Created: 30.04.2015 15:45:51
 *  Author: Kjetil
 */ 

#include <sam.h>
#include <arm_math.h>


float angleConversions_encoderToDegrees(uint8_t mostSignificantBitNumber, uint32_t encoderValue)
{
	uint32_t encoderMaxValue = ( 1 << mostSignificantBitNumber ) - 1;
	return ((float)(encoderValue * 360))/((float) encoderMaxValue);
}

float angleConversions_encoderToEletricalDegrees(uint8_t mostSignificantBitNumber, int32_t encoderValue, uint8_t polePairs)
{
	int32_t encoderMaxValue = ((1 << mostSignificantBitNumber) - 1);
	return ((float) ( ((encoderValue * polePairs) % encoderMaxValue) * 360 ) )/((float) encoderMaxValue);	
}

float angleConversions_encoderToRPM(int32_t encoderValue, uint32_t encoderMax, int32_t sampleFrequency)
{
	static float rpmEstimate;
	static uint32_t smoothedEncoderValue;
	float rawRpmEstimate;
	int32_t deltaEncoderValue;
	
	
	deltaEncoderValue = (encoderValue - smoothedEncoderValue);
	if(deltaEncoderValue > (int32_t)(encoderMax / 2)){
		deltaEncoderValue -= encoderMax;
		deltaEncoderValue = -deltaEncoderValue;
	}
	else if(deltaEncoderValue < (-((int32_t)encoderMax/2))){
		deltaEncoderValue += encoderMax;
		deltaEncoderValue = -deltaEncoderValue;
	}
	rawRpmEstimate = ((float)deltaEncoderValue) * ((float)sampleFrequency) * 60.0 / (float)encoderMax;
	rpmEstimate = rpmEstimate * 0.7 + 0.3*rawRpmEstimate; // needs tuning
	
	//smoothedEncoderValue = (smoothedEncoderValue >> 1) + (encoderValue >> 1);
	smoothedEncoderValue = encoderValue;
	
	return rpmEstimate;
}