/*
 * can_wrapper.c
 *
 * Created: 08.03.2016 21.02.01
 *  Author: Simen
 */ 

#include <sam.h>
#include <arm_math.h>
#include <stdbool.h>
#include "system_state.h"
#include "CAN-IDs/revolve_can_definitions.h"
#include "Sam-base/RevolveDrivers/CAN/can.h"
#include "inverter_IDs.h"
#include "can_wrapper.h"
#include "parameter_store.h"
#include "status.h"


struct Can_message_t inverter_alive_msg = {
	.dataLength = 2,
	.idType = STANDARD_CAN_ID,
	.messageID = CANR_ALIVE_MSG_ID,
	.data.u8[0] = INVERTER_ALIVE_ID
};

struct Can_message_t inverter_mech_msg = {
	.dataLength = 8,
	.idType = STANDARD_CAN_ID,
	.messageID = R16_DATA_MECHANICAL_ID(0),
};

struct Can_message_t inverter_electrical_msg = {
	.dataLength = 8,
	.idType = STANDARD_CAN_ID,
	.messageID = R16_DATA_ELECTRICAL_ID(0),
};

struct Can_message_t inverter_status_msg = {
	.dataLength = 8,
	.idType = STANDARD_CAN_ID,
	.messageID = R16_STATUS_ID(0),
};

struct Can_message_t inverter_parameter_msg = {
	.dataLength = 6,
	.idType = STANDARD_CAN_ID,
	.messageID = R16_PARAMETERS
};

struct Can_message_t* can_transmit_buffers[NUM_DATA_MESSAGES] = {
	&inverter_mech_msg,
	&inverter_electrical_msg,
};

volatile float32_t external_max_torque_request = 0;
volatile float32_t external_min_torque_request = 0;
volatile float32_t external_speed_request = 0;

void can_callback(struct Can_message_t *recieve_buffer){
	switch(recieve_buffer->messageID){
		case PARAMETER_SET_ID:
			if(recieve_buffer->data.u8[0] == CANR_GRP_INVERTER_R16_ID){
				if(!status_check(INVERTER_ENABLED | READY_TO_DRIVE)){
					parameter_set(recieve_buffer->data.u8[PARAMETER_SET_PARAM_ID_8pos], recieve_buffer->data.u32[PARAMETER_SET_PARAM_val_32pos]);
				}
			}
		break;
		case STATUS_BIT_SET_ID:
			if(recieve_buffer->data.u8[STATUS_BIT_SET_MODULE_ID_8pos] == CANR_GRP_INVERTER_R16_ID){
				switch(recieve_buffer->data.u8[STATUS_BIT_SET_OPERATION_ID_8pos]){
					case STATUS_BIT_SET_OPERATION_SET:
					status_signal(recieve_buffer->data.u32[STATUS_BIT_SET_PARAM_mask_32pos]);
					break;
					case STATUS_BIT_SET_OPERATION_CLEAR:
					status_clear(recieve_buffer->data.u32[STATUS_BIT_SET_PARAM_mask_32pos]);
					break;
					case STATUS_BIT_SET_OPERATION_FLIP:
					status_toggle(recieve_buffer->data.u32[STATUS_BIT_SET_PARAM_mask_32pos]);
					break;
					default:
					break;
				}
			
			}
			break;
		case 0:
			if(!status_check(ACCEPT_TORQUE_REQUEST_FROM_TELEMETRY_NOT_ECU)){
				external_speed_request = (float32_t)recieve_buffer->data.i16[1]		 / 100;
				external_max_torque_request = (float32_t)recieve_buffer->data.i16[2] / 100;
				external_min_torque_request = (float32_t)recieve_buffer->data.i16[3] / 100;
			}
		break;
		case R16_TELEMETRY_TORQUE_REQUEST_ID:
		if(recieve_buffer->data.u8[0] == CANR_GRP_INVERTER_R16_ID){
			if(status_check(ACCEPT_TORQUE_REQUEST_FROM_TELEMETRY_NOT_ECU)){
				external_speed_request = (float32_t)recieve_buffer->data.i16[1]		 / 100;
				external_max_torque_request = (float32_t)recieve_buffer->data.i16[2] / 100;
				external_min_torque_request = (float32_t)recieve_buffer->data.i16[3] / 100;
			}
		}
		break;
	case 0x5a5:
	case 0x67B:
			status_toggle( (1 << recieve_buffer->data.u8[2]) );
	break;
	}
}

void can_callback_0(uint32_t index){
	struct Can_message_t recieve_buffer;
	
	can_getMessage(MCAN0, &recieve_buffer, index);
	
	can_callback(&recieve_buffer);
}

void can_callback_1(uint32_t index){
	struct Can_message_t recieve_buffer;
	
	can_getMessage(MCAN1, &recieve_buffer, index);
	
	can_callback(&recieve_buffer);
}

uint32_t can_receive_IDS[6] = {
	PARAMETER_SET_ID,
	STATUS_BIT_SET_ID,
	0,
	R16_TELEMETRY_TORQUE_REQUEST_ID,
	0x5a5,
	0x67B
};

void can_wrapper_init(uint32_t inverter_ID){
	can_init(MCAN1, CAN_BPS_1000K, can_receive_IDS , 6, NULL, 0, can_callback_1 , 1);
	can_init(MCAN0, CAN_BPS_1000K, can_receive_IDS , 6, NULL, 0, can_callback_0 , 1);
	
}

float32_t can_wrapper_get_max_torque_request(){
	return external_max_torque_request;
}

float32_t can_wrapper_get_min_torque_request(){
	return external_min_torque_request;
}

float32_t can_wrapper_get_speed_request(){
	return external_speed_request;
}

bool can_wrapper_send_data_msg(volatile state_variables_t *state){
	//Send state data @100hz
	static uint8_t can_transmit_iterator = 0;
	if(can_transmit_buffers[can_transmit_iterator]->messageID == R16_DATA_MECHANICAL_ID(0)){
	//	can_transmit_buffers[can_transmit_iterator]->data.f[0] = (state->speed);
		can_transmit_buffers[can_transmit_iterator]->data.i16[1] = (uint16_t)(state->speed);
		can_transmit_buffers[can_transmit_iterator]->data.u32[1] = (state->encoder_data);
		can_transmit_buffers[can_transmit_iterator]->data.i16[0] = (int16_t)(state->torque_request * 100.0);
	}
	else if(can_transmit_buffers[can_transmit_iterator]->messageID == R16_DATA_ELECTRICAL_ID(0)){
		
		/*can_transmit_buffers[can_transmit_iterator]->data.f[0] = (state->current_q);
		can_transmit_buffers[can_transmit_iterator]->data.f[1] = (state->current_d);
		*/
		can_transmit_buffers[can_transmit_iterator]->data.i16[0] = (int16_t)(state->current_q * 100.0);
		can_transmit_buffers[can_transmit_iterator]->data.i16[1] = (int16_t)(state->current_d * 100.0);
		can_transmit_buffers[can_transmit_iterator]->data.i16[2] = (int16_t)(state->current_a * 100.0);
		can_transmit_buffers[can_transmit_iterator]->data.i16[3] = (int16_t)(state->current_b * 100.0);
	}
	
	if(can_sendMessage(MCAN1, can_transmit_buffers[can_transmit_iterator])== STATUS_OK){
		can_transmit_iterator = (can_transmit_iterator + 1) % NUM_DATA_MESSAGES;
		return true;
	}
	else{
		return false;
	}
}

void can_wrapper_send_parameter(uint32_t inverter_ID, uint8_t parameter_ID, int16_t parameter_data){
	inverter_parameter_msg.data.u8[1] = inverter_ID;
	inverter_parameter_msg.data.u8[0] = parameter_ID;
	inverter_parameter_msg.data.i16[1] = parameter_data;
	
	while(can_sendMessage(MCAN1, &inverter_parameter_msg) != STATUS_OK);
}

bool can_wrapper_send_state_msg(){
	inverter_status_msg.data.u32[0] = status_getVector();
	inverter_status_msg.data.u32[1] = status_getLastVector();
	return (can_sendMessage(MCAN1, &inverter_status_msg) == STATUS_OK);
}

bool can_wrapper_send_alive_msg(){
	return (can_sendMessage(MCAN0, &inverter_alive_msg) == STATUS_OK);
}