/*
 * errors.c
 *
 * Created: 27.06.2016 16.32.37
 *  Author: Simen
 */ 

#include <sam.h>
#include <stdlib.h>
#include <arm_math.h>
#include <stdio.h>
#include <math.h>
#include <stdbool.h>
#include "utilities.h"
#include "system_state.h"
#include "Sam-base/RevolveDrivers/pio.h"

#include "Drivers/mazet_status.h"

#include "statusCodes.h"


#define COPROCESSOR_ERROR_PIO	PIOD
#define COPROCESSOR_TEMP_PIN	8

bool check_persistent_error(uint32_t *counter, uint32_t period, float32_t val, float32_t limit){
	if(fabsf(val) > limit){
		*counter = (*counter + 1) > period ? period : *counter + 1;
	}
	else{
		*counter = (*counter > 0) ? *counter - 1 : 0;
	}
	return (*counter > (period / 2));
}


void check_errors(state_variables_t *state_data){
	static uint8_t error_check_iterator = 0;
	static uint32_t q_counter = 0;
	static uint32_t d_counter = 0;
	static uint32_t Vdc_counter = 0;
	
	switch(error_check_iterator){
		case 0:
		if(fabsf(state_data->current_a) > CURRENT_PHASE_LIMIT_TRANSIENT){
			if(fabsf(state_data->current_a) > CURRENT_PHASE_LIMIT_PERMANENT){
				status_signal(CURRENT_LIMIT_EXCEEDED_PERMANENT);
			}
			status_signal(CURRENT_LIMIT_EXCEEDED_PHASE_A_TRANSIENT);
		}
		else{
			status_clear(CURRENT_LIMIT_EXCEEDED_PHASE_A_TRANSIENT);
		}
		break;
		case 1:
		if(fabsf(state_data->current_b) > CURRENT_PHASE_LIMIT_TRANSIENT){
			if(fabsf(state_data->current_b) > CURRENT_PHASE_LIMIT_PERMANENT){
				status_signal(CURRENT_LIMIT_EXCEEDED_PERMANENT);
			}
			status_signal(CURRENT_LIMIT_EXCEEDED_PHASE_B_TRANSIENT);
		}
		else{
			status_clear(CURRENT_LIMIT_EXCEEDED_PHASE_B_TRANSIENT);
		}
		break;
		case 2:
		if(fabsf(state_data->current_a - state_data->current_b) > CURRENT_PHASE_LIMIT_TRANSIENT){
			if(fabsf(state_data->current_a - state_data->current_b) > CURRENT_PHASE_LIMIT_PERMANENT){
				status_signal(CURRENT_LIMIT_EXCEEDED_PERMANENT);
			}
			status_signal(CURRENT_LIMIT_EXCEEDED_PHASE_C_TRANSIENT);
		}
		else{
			status_clear(CURRENT_LIMIT_EXCEEDED_PHASE_C_TRANSIENT);
		}
		break;
		case 3:
		if(fabsf(state_data->voltage_dc) > VOLTAGE_DC_MAX_LIMIT_PERMANENT){
			status_signal(VOLTAGE_LIMIT_EXCEEDED_PERMANENT);
		}
		if(check_persistent_error(&Vdc_counter, MAX_PERSISTENT_ERRORS_VDC, state_data->voltage_dc, VOLTAGE_DC_MAX_LIMIT_TRANSIENT)){
			
			status_signal(VOLTAGE_LIMIT_EXCEEDED_TRANSIENT);
		}
		else{
			status_clear(VOLTAGE_LIMIT_EXCEEDED_TRANSIENT);
		}
		break;
		case 4:
		if(state_data->voltage_dc > VOLTAGE_DC_MIN_LIMIT_TRANSIENT){
			status_signal(VOLTAGE_ABOVE_THRESHOLD);
		}
		else{
			status_clear(VOLTAGE_ABOVE_THRESHOLD);
		}
		break;
		case 5:
		if(state_data->speed > RPM_LIMIT_TRANSIENT){
			status_signal(SPEED_LIMIT_EXCEEDED_TRANSIENT);
		}
		else{
			status_clear(SPEED_LIMIT_EXCEEDED_TRANSIENT);
		}
		break;
		case 6:
		if(pio_readPin(COPROCESSOR_ERROR_PIO, COPROCESSOR_TEMP_PIN)){
			status_signal(TEMP_LIMIT_EXCEEDED_TRANSIENT(TEMP_ALL));
		}
		break;
		case 7:
		if(state_data->encoder_status & MAZET_STATUS_ERROR_mask){
			status_signal(ENCODER_ERROR);
		}
		else{
			status_clear(ENCODER_ERROR);
		}
		if(state_data->encoder_status & MAZET_STATUS_WARNING_mask){
			status_signal(ENCODER_WARNING);
		}
		else{
			status_clear(ENCODER_WARNING);
		}
		if(state_data->encoder_status & MAZET_STATUS_ERROR_1_bit){
			status_signal(ENCODER_ERROR_1);
		}
		else{
			status_clear(ENCODER_ERROR_1);
		}
		if(state_data->encoder_status & MAZET_STATUS_ERROR_2_bit){
			status_clear(ENCODER_ERROR_2);
		}
		else{
			status_signal(ENCODER_ERROR_2);
		}
		break;
		case 8:
		if(check_persistent_error(&q_counter, MAX_PERSISTENT_ERRORS_DQ, state_data->current_q, CURRENT_PHASE_LIMIT_TRANSIENT)){
			status_signal(CURRENT_LIMIT_EXCEEDED_Q_TRANSIENT);
		}
		else{
			status_clear(CURRENT_LIMIT_EXCEEDED_Q_TRANSIENT);
		}
		break;
		case 9:
		if(check_persistent_error(&d_counter, MAX_PERSISTENT_ERRORS_DQ, state_data->current_d, CURRENT_D_LIMIT_TRANSIENT)){
			status_signal(CURRENT_LIMIT_EXCEEDED_D_TRANSIENT);
		}
		else{
			status_clear(CURRENT_LIMIT_EXCEEDED_D_TRANSIENT);
		}
		break;
	}
	error_check_iterator = (error_check_iterator + 1) % 10;
}
