/*
 * exceptionHandler.c
 *
 * Created: 17.03.2016 21:19:18
 *  Author: simtind
 */ 

#include <sam.h>

void printErrorMsg(const char *errmsg){
	while(*errmsg != '\0'){
		ITM_SendChar(*errmsg);
		++errmsg;
	}
}

void NMI_Handler        ( void ) {
	while(1);
}


/**
 * \brief This function back trace the stack to give exact address where fault
 happened
**/
static uint32_t StackUnwind(void)
{
        volatile uint32_t Fault_Add;
        asm volatile ("mrs r0, msp ");
        asm volatile ("ldr %0, [r0,#28]" : "=r" (Fault_Add));
        return Fault_Add;
}

typedef enum{
	HARDFAULT_DEBUG,
	HARDFAULT_VECTOR_TABLE_READ,
	HARDFAULT_FORCED_USAGE_DIV_BY_ZERO,
	HARDFAULT_FORCED_USAGE_UNALIGNED_ACCESS,
	HARDFAULT_FORCED_USAGE_COPROCESSOR_ACCESS,
	HARDFAULT_FORCED_USAGE_INTEGRITY_CHECK,
	HARDFAULT_FORCED_BUS_PRECISE_ACCESS,
	HARDFAULT_FORCED_BUS_EXCEPTION_ENTRY,
	HARDFAULT_FORCED_BUS_EXCEPTION_RETURN,
	HARDFAULT_FORCED_BUS_IMPRECISE_ACCESS,
	HARDFAULT_FORCED_BUS_INSTRUCTION_PREFETCH,
	HARDFAULT_FORCED_MEMORY_DIV_BY_ZERO	
}Hardfault_reason_t;

/**
 * \brief If Other Faults are enabled then HardFault error will look for those
 *  errors to give more detail about fault
**/
Hardfault_reason_t HardFault_reason(void)
{
    uint32_t CFSRValue;
	volatile Hardfault_reason_t reason;

    if ((SCB->HFSR & SCB_HFSR_DEBUGEVT_Msk)) {
		reason = HARDFAULT_DEBUG;
    }

    if ((SCB->HFSR & SCB_HFSR_VECTTBL_Msk)) {
		reason = HARDFAULT_VECTOR_TABLE_READ;
    }

    // Forced HardFault
	    if ((SCB->HFSR & SCB_HFSR_FORCED_Msk)) {
	            // Usage Fault
	            if ((SCB->CFSR & SCB_CFSR_USGFAULTSR_Msk)) {
	                    CFSRValue = SCB->CFSR;
	                    CFSRValue >>= SCB_CFSR_USGFAULTSR_Pos;

	                    if ((CFSRValue & (1 << 9)))
							reason = HARDFAULT_FORCED_USAGE_DIV_BY_ZERO;

	                    if ((CFSRValue & (1 << 8)))
							reason = HARDFAULT_FORCED_USAGE_UNALIGNED_ACCESS;

	                    if ((CFSRValue & (1 << 3)))
							reason = HARDFAULT_FORCED_USAGE_COPROCESSOR_ACCESS;

	                    if ((CFSRValue & (1 << 2)))
							reason = HARDFAULT_FORCED_USAGE_INTEGRITY_CHECK;
	            }
	            // Bus Fault
	            if ((SCB->CFSR & SCB_CFSR_BUSFAULTSR_Msk)) {
	                    CFSRValue = SCB->CFSR;
	                    CFSRValue >>= SCB_CFSR_BUSFAULTSR_Pos;

	                    if ((CFSRValue & (1 << 7)) && (CFSRValue & (1 << 1)))
		                    reason = HARDFAULT_FORCED_BUS_PRECISE_ACCESS;
	                    if ((CFSRValue & (1 << 4)))
							reason = HARDFAULT_FORCED_BUS_EXCEPTION_ENTRY;

	                    if ((CFSRValue & (1 << 3)))
	                        reason = HARDFAULT_FORCED_BUS_EXCEPTION_RETURN;

	                    if ((CFSRValue & (1 << 2)))
							reason = HARDFAULT_FORCED_BUS_IMPRECISE_ACCESS;

	                    if ((CFSRValue & (1 << 0))) {
		                    reason = HARDFAULT_FORCED_BUS_INSTRUCTION_PREFETCH;
	                    }
	            }
	    }

	    // MemoryFault
	    if ((SCB->CFSR & SCB_CFSR_MEMFAULTSR_Msk)) {
	            CFSRValue = SCB->CFSR;
	            CFSRValue >>= SCB_CFSR_MEMFAULTSR_Pos;

	            if ((CFSRValue & (1 << 9)) != 0)
					reason = HARDFAULT_FORCED_MEMORY_DIV_BY_ZERO;
	    }

	    __ISB();
	    __DMB();
		return reason;
}

void prvGetRegistersFromStack( uint32_t pulFaultStackAddress[] )
{
	/* These are volatile to try and prevent the compiler/linker optimising them
	away as the variables never actually get used.  If the debugger won't show the
	values of the variables, make them global my moving their declaration outside
	of this function. */
	static volatile __attribute__((unused)) uint32_t r0;
	static volatile __attribute__((unused)) uint32_t r1;
	static volatile __attribute__((unused)) uint32_t r2;
	static volatile __attribute__((unused)) uint32_t r3;
	static volatile __attribute__((unused)) uint32_t r12;
	static volatile __attribute__((unused)) uint32_t lr; /* Link register. */
	static volatile __attribute__((unused)) uint32_t pc_val; /* Program counter. */
	static volatile __attribute__((unused)) uint32_t psr;/* Program status register. */
	static volatile __attribute__((unused)) uint32_t hfsr;
	
	r0 = pulFaultStackAddress[ 0 ];
	r1 = pulFaultStackAddress[ 1 ];
	r2 = pulFaultStackAddress[ 2 ];
	r3 = pulFaultStackAddress[ 3 ];
	r12 = pulFaultStackAddress[ 4 ];
	lr = pulFaultStackAddress[ 5 ];
	pc_val = pulFaultStackAddress[ 6 ];
	psr = pulFaultStackAddress[ 7 ];
	
}


void HardFault_Handler  ( void )  {
/*
	asm volatile
    (
        " tst lr, #4                                                \n"
        " ite eq                                                    \n"
        " mrseq r0, msp                                             \n"
        " mrsne r0, psp                                             \n"
        " ldr r1, [r0, #24]                                         \n"
        " ldr r2, handler2_address_const                            \n"
        " bx r2                                                     \n"
        " handler2_address_const: .word prvGetRegistersFromStack    \n"
    );*/
	
	__ISB();
	__DMB();
	
	volatile uint32_t __attribute__((unused)) program_counter = StackUnwind();
	volatile Hardfault_reason_t __attribute__((unused)) reason = HardFault_reason();
	
	asm volatile("BKPT #01");
	while(1){	}
}
void MemManage_Handler  ( void )  {
	while(1);
}
void BusFault_Handler   ( void )  {
	while(1);
}
void UsageFault_Handler ( void )  {
	while(1);
}
void SVC_Handler        ( void )  {
	while(1);
}
void DebugMon_Handler   ( void )  {
	while(1);
}
void PendSV_Handler     ( void )  {
	while(1);
}
void SysTick_Handler    ( void )  {
	while(1);
}