/*
 * parameter_store.c
 *
 * Created: 07.03.2016 20.01.05
 *  Author: Simen
 */ 

#include <sam.h>
#include <arm_math.h>
#include <stdbool.h>
#include "system_state.h"
#include "can_wrapper.h"
#include "Sam-base/RevolveDrivers/Flash/flash_userPage.h"
#include "Motor_control/pid.h"
#include "parameter_store.h"

const float32_t default_parameters[NUM_PARAMETERS] = {
	PARAMETER_VALID_DATA_code,
	1.0,
	0.04,
	0.0,
	0.0,
	1.0,
	0.04,
	0.0,
	0.0,
	0.01,
	0.01,
	0,
};

float32_t parameters[NUM_PARAMETERS] = {0};
	
float32_t parameter_get(uint8_t parameter_ID){
	return parameters[parameter_ID];
}

void parameters_write_new_data(){
	flash_erase_user_signature();
	flash_write_user_signature((flash_data_t *)parameters, NUM_PARAMETERS);
}

void parameters_get_stored_data(){
	flash_read_user_signature((flash_data_t *)parameters, NUM_PARAMETERS);
	
	if(parameters[PARAMETER_VALID_DATA] != PARAMETER_VALID_DATA_code){
		memcpy(parameters, default_parameters, NUM_PARAMETERS * sizeof(float32_t));
		parameters_write_new_data();
	}
}


bool parameter_check_and_set(float32_t *old_parameter, float32_t new_parameter){
	if(*old_parameter != new_parameter){
		*old_parameter = new_parameter;
		return true;
	}
	else{
		return false;
	}
}

void parameter_set( uint8_t parameter_id, float32_t parameter_data){
	bool update_data = false;
	
	update_data = parameter_check_and_set(&(parameters[parameter_id]), parameter_data);
	
	if (update_data){
		parameters_write_new_data(parameters);
	}
}

void parameters_update_and_transmit(state_variables_t *state){
	parameters_get_stored_data();
	
	state->regulator_data.field_weakening_gain_up			= 1.0; //= parameter_get(PARAMETER_FW_K_p_up);
	state->regulator_data.field_weakening_gain_down			= 1.0; //= parameter_get(PARAMETER_FW_K_p_dn);
		 													//
	state->regulator_data.PID_reg_D.settings.K_p			= 1.0; //	= parameter_get(PARAMETER_D_K_p);
	state->regulator_data.PID_reg_D.settings.K_i			= 0.05; //	= parameter_get(PARAMETER_D_K_i);
	state->regulator_data.PID_reg_D.settings.K_d			= 0.0; //	= parameter_get(PARAMETER_D_K_d);
	state->regulator_data.PID_reg_D.settings.K_antiwindup	= 0.0; //= parameter_get(PARAMETER_D_K_a);
		 													//
	state->regulator_data.PID_reg_Q.settings.K_p			= 1.1; //	= parameter_get(PARAMETER_Q_K_p);
	state->regulator_data.PID_reg_Q.settings.K_i			= 0.05;//	= parameter_get(PARAMETER_Q_K_i);
	state->regulator_data.PID_reg_Q.settings.K_d			= 0.0; //	= parameter_get(PARAMETER_Q_K_d);
	state->regulator_data.PID_reg_Q.settings.K_antiwindup   = 0.0; //= parameter_get(PARAMETER_Q_K_a);
	
	can_wrapper_send_parameter(parameter_get(PARAMETER_INVID), PARAMETER_FW_K_p_up, (int16_t)(state->regulator_data.field_weakening_gain_up			* 100));
	can_wrapper_send_parameter(parameter_get(PARAMETER_INVID), PARAMETER_FW_K_p_dn, (int16_t)(state->regulator_data.field_weakening_gain_down		* 100));
	can_wrapper_send_parameter(parameter_get(PARAMETER_INVID), PARAMETER_D_K_p,		(int16_t)(state->regulator_data.PID_reg_D.settings.K_p			* 100));
	can_wrapper_send_parameter(parameter_get(PARAMETER_INVID), PARAMETER_D_K_i,		(int16_t)(state->regulator_data.PID_reg_D.settings.K_i			* 100));
	can_wrapper_send_parameter(parameter_get(PARAMETER_INVID), PARAMETER_D_K_d,		(int16_t)(state->regulator_data.PID_reg_D.settings.K_d			* 100));
	can_wrapper_send_parameter(parameter_get(PARAMETER_INVID), PARAMETER_D_K_a,		(int16_t)(state->regulator_data.PID_reg_D.settings.K_antiwindup	* 100));
	can_wrapper_send_parameter(parameter_get(PARAMETER_INVID), PARAMETER_Q_K_p,		(int16_t)(state->regulator_data.PID_reg_Q.settings.K_p			* 100));
	can_wrapper_send_parameter(parameter_get(PARAMETER_INVID), PARAMETER_Q_K_i,		(int16_t)(state->regulator_data.PID_reg_Q.settings.K_i			* 100));
	can_wrapper_send_parameter(parameter_get(PARAMETER_INVID), PARAMETER_Q_K_d,		(int16_t)(state->regulator_data.PID_reg_Q.settings.K_d			* 100));
	can_wrapper_send_parameter(parameter_get(PARAMETER_INVID), PARAMETER_Q_K_a,		(int16_t)(state->regulator_data.PID_reg_Q.settings.K_antiwindup	* 100));
}