/*
 * status.c
 *
 * Created: 26.04.2015 00:53:09
 *  Author: Kjetil
 */ 

#include <stdbool.h>
#include <stdint.h>

#include "status.h"
#include "statusCodes.h"


static bool initiated = false;
static uint32_t statusCounter;
static uint32_t recoverableStatusVector;
static uint32_t statusVector, accumulatedStatusVector;

int32_t status_init(uint32_t recoverableStatusBitMask)
{
    if(initiated)
		return -1;
    recoverableStatusVector = recoverableStatusBitMask;
    statusVector = 0;
	accumulatedStatusVector = 0;
    initiated = true;
    return 0;
}

void status_signal(uint32_t statusBitMask)
{
    statusVector |= statusBitMask;
	accumulatedStatusVector |= statusVector;
	statusCounter++;
}

void status_toggle(uint32_t statusBitMask){
	if(status_check(statusBitMask)){
		status_clear(statusBitMask);
	}
	else{
		status_signal(statusBitMask);
	}
}

void status_clear(uint32_t statusBitMask)
{
    uint32_t recoverVector = statusBitMask & recoverableStatusVector;
    statusVector &= ~(recoverVector);
}

void status_clear_tansient_errors(){
	status_clear(TRANSIENT_ERRORS);
}

bool status_check(uint32_t statusBitMask)
{
    return ((bool)(statusVector & statusBitMask));
}

uint32_t status_getVector()
{
	return statusVector;
}

uint32_t status_getLastVector()
{
	return accumulatedStatusVector;
}

uint32_t status_getCount()
{
	return statusCounter;
}
