/*
 * statusBits.h
 *
 * Created: 08.05.2015 20:56:27
 *  Author: Kjetil
 */ 


#ifndef STATUSBITS_H_
#define STATUSBITS_H_

// List of permanent shutdown limits
#define CURRENT_PHASE_LIMIT_PERMANENT 200.0
#define VOLTAGE_DC_MAX_LIMIT_PERMANENT 700.0

#define MAX_TRANSIENT_ERRORS 3
#define MAX_PERSISTENT_ERRORS_DQ 250
#define MAX_PERSISTENT_ERRORS_VDC 100

// List of transient shutdown limits
#define CURRENT_PHASE_LIMIT_TRANSIENT RMS(100.0)
#define CURRENT_D_LIMIT_TRANSIENT 70.0
#define VOLTAGE_DC_MAX_LIMIT_TRANSIENT 600.0
#define VOLTAGE_DC_MIN_LIMIT_TRANSIENT 45.0
#define RPM_LIMIT_TRANSIENT (20000.0)
#define TORQUE_REQUEST_LOWER_LIMIT_TRANSIENT 0.1
#define RPM_HYSTERESIS 100.0

// List of all possible errors
#define CURRENT_LIMIT_EXCEEDED_PERMANENT				(1 << 0)
#define VOLTAGE_LIMIT_EXCEEDED_PERMANENT				(1 << 1)

#define CURRENT_LIMIT_EXCEEDED_PHASE_A_TRANSIENT		(1 << 3)
#define CURRENT_LIMIT_EXCEEDED_PHASE_B_TRANSIENT		(1 << 4)
#define CURRENT_LIMIT_EXCEEDED_PHASE_C_TRANSIENT		(1 << 5)
#define CURRENT_LIMIT_EXCEEDED_Q_TRANSIENT				(1 << 6)
#define CURRENT_LIMIT_EXCEEDED_D_TRANSIENT				(1 << 7)
#define VOLTAGE_LIMIT_EXCEEDED_TRANSIENT				(1 << 8)
#define SPEED_LIMIT_EXCEEDED_TRANSIENT					(1 << 9)
#define TEMP_LIMIT_EXCEEDED_TRANSIENT(id)				(id << 10)
	#define TEMP_1		1
	#define TEMP_2		2
	#define TEMP_3		4
	#define TEMP_ALL	7
	
#define VOLTAGE_ABOVE_THRESHOLD							(1<<15)
//Controller state
#define PID_SATURATION_D								(1 << 17)
#define PID_SATURATION_Q								(1 << 18)
#define FOC_IN_FIELD_WEAKENING							(1 << 19)
#define FOC_VOLTAGE_SATURATION							(1 << 20)
#define LOOP_LOST_SYNC									(1 << 21)

#define ENCODER_ERROR									(1 << 23)
#define ENCODER_WARNING									(1 << 24)
#define ENCODER_ERROR_1									(1 << 25)
#define ENCODER_ERROR_2									(1 << 26)

// switchable status bits
#define INVERTER_ENABLED								(1 << 28)
#define READY_TO_DRIVE									(1 << 29)
#define ACCEPT_TORQUE_REQUEST_FROM_TELEMETRY_NOT_ECU	(1 << 30)
#define OVERRIDE_SPEED_CONTROLLER						(1 << 31)

// This is permanent (unrecoverable) errors
#define PERMANENT_ERRORS (CURRENT_LIMIT_EXCEEDED_PERMANENT|VOLTAGE_LIMIT_EXCEEDED_PERMANENT)

#define ENCODER_FAILURE (ENCODER_ERROR									\
						|ENCODER_WARNING								\
						|ENCODER_ERROR_1								\
						|ENCODER_ERROR_2								)

//  This is the transient errors
#define TRANSIENT_ERRORS (	 CURRENT_LIMIT_EXCEEDED_PHASE_A_TRANSIENT	\
							|CURRENT_LIMIT_EXCEEDED_PHASE_B_TRANSIENT	\
							|CURRENT_LIMIT_EXCEEDED_PHASE_C_TRANSIENT	\
							|VOLTAGE_LIMIT_EXCEEDED_TRANSIENT			\
							|SPEED_LIMIT_EXCEEDED_TRANSIENT				\
							|CURRENT_LIMIT_EXCEEDED_Q_TRANSIENT			\
							|CURRENT_LIMIT_EXCEEDED_D_TRANSIENT			\
							|ENCODER_FAILURE)
						/*|TEMP_LIMIT_EXCEEDED_TRANSIENT(TEMP_ALL)	*/
						

#define DERATING_ERRORS (TEMP_LIMIT_EXCEEDED_TRANSIENT(TEMP_ALL))

// This is the status, not really errors
#define STATUS_BITS (	 READY_TO_DRIVE									\
						|INVERTER_ENABLED								\
						|ACCEPT_TORQUE_REQUEST_FROM_TELEMETRY_NOT_ECU	\
						|PID_SATURATION_D								\
						|PID_SATURATION_Q								\
						|FOC_IN_FIELD_WEAKENING							\
						|FOC_VOLTAGE_SATURATION							\
						|OVERRIDE_SPEED_CONTROLLER						\
						|VOLTAGE_ABOVE_THRESHOLD						\
						|LOOP_LOST_SYNC									\
					)

#endif /* STATUSBITS_H_ */