/******************************************************************************
*
* Copyright (C) 2009 - 2014 Xilinx, Inc.  All rights reserved.
*
* Permission is hereby granted, free of charge, to any person obtaining a copy
* of this software and associated documentation files (the "Software"), to deal
* in the Software without restriction, including without limitation the rights
* to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
* copies of the Software, and to permit persons to whom the Software is
* furnished to do so, subject to the following conditions:
*
* The above copyright notice and this permission notice shall be included in
* all copies or substantial portions of the Software.
*
* Use of the Software is limited solely to applications:
* (a) running on a Xilinx device, or
* (b) that interact with a Xilinx device through a bus or interconnect.
*
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
* XILINX  BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
* WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF
* OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
* SOFTWARE.
*
* Except as contained in this notice, the name of the Xilinx shall not be used
* in advertising or otherwise to promote the sale, use or other dealings in
* this Software without prior written authorization from Xilinx.
*
******************************************************************************/

/*
 * helloworld.c: simple test application
 *
 * This application configures UART 16550 to baud rate 9600.
 * PS7 UART (Zynq) is not initialized by this application, since
 * bootrom/bsp configures it to baud rate 115200
 *
 * ------------------------------------------------
 * | UART TYPE   BAUD RATE                        |
 * ------------------------------------------------
 *   uartns550   9600
 *   uartlite    Configurable only in HW design
 *   ps7_uart    115200 (configured by bootrom/bsp)
 */

#include <stdio.h>
#include "platform.h"
#include "xparameters.h"
#include "xil_printf.h"
#include "sleep.h"
#include "xil_types.h"
#include "xil_exception.h"

#include "xgpio.h"
#include "xadcps.h"
#include "xcanps.h"

#include "xscugic.h"
#include "xtmrctr.h"




/**** GPIO definitions ****/

#define GPIO_OUT_CH 1
#define GPIO_IN_CH  2

XGpio Gpio;

/********* XADC definitions ********/

#define XPAR_AXI_XADC_0_DEVICE_ID 0

static XAdcPs XADCMonInst;

int XADCInit(XAdcPs* XADCInstPtr);

/************ CAN definitions **************/
#define CAN_DEVICE_ID XPAR_XCANPS_0_DEVICE_ID
#define XCANPS_MAX_FRAME_SIZE_IN_WORDS (XCANPS_MAX_FRAME_SIZE / sizeof(u32))
#define FRAME_DATA_LENGTH 		8
#define TEST_MESSAGE_ID			2000

#define TEST_BTR_SYNCJUMPWIDTH		3
#define TEST_BTR_SECOND_TIMESEGMENT	3
#define TEST_BTR_FIRST_TIMESEGMENT	6

#define TEST_BRPR_BAUD_PRESCALAR	1
int CanInit(XCanPs* CanInstPtr, u16 DeviceID);
static int SendFrame(XCanPs *InstancePtr);
static int RecvFrame(XCanPs *InstancePtr);

static u32 TxFrame[XCANPS_MAX_FRAME_SIZE_IN_WORDS];
static u32 RxFrame[XCANPS_MAX_FRAME_SIZE_IN_WORDS];

static XCanPs Can;

/*********** TmrCtr definitions ***************/
#define TMRCTR_0_DEVICE_ID 	XPAR_AXI_TIMER_0_DEVICE_ID
#define TMRCTR_1_DEVICE_ID 	XPAR_AXI_TIMER_1_DEVICE_ID
#define TMRCTR_INT_ID		XPS_FPGA1_INT_ID


#define INTC_DEVICE_ID		XPAR_SCUGIC_SINGLE_DEVICE_ID

#define INTC 				XScuGic
#define INTC_HANDLER 		XScuGic_InterruptHandler
#define TIMER_COUNTER_0 	0
#define TIMER_COUNTER_1		1

INTC InterruptController;
XTmrCtr TC0;
XTmrCtr TC1;

#define RESET_VALUE0		0xFD050F7F

#define PWM_AXI_CLK_HZ		50000000

void PWMSetPeriod(XTmrCtr *TCPtr, float pwmFreq);
void PWMSetDuty(XTmrCtr *TCPtr, float Duty);
int  PWMInit(XTmrCtr *TCPtr, u32 DeviceID);

//Function to be called on TC timeout
void TC0Handler();
int TCInit(XTmrCtr *TCPtr, u32 DeviceID, XTmrCtr_Handler funcPtr, u8 TCNum, u32 reset_val);

/************/
int main(){
	xil_printf("Hello World\n\r");
	u32 gpio_input;
	int Status;



	/******* Initialize AXI GPIO **********/
	XGpio_Initialize(&Gpio, XPAR_GPIO_0_DEVICE_ID);
	XGpio_DiscreteSet(&Gpio, GPIO_OUT_CH, 0); // Clear LEDs.

	/******* Initialize XADC **********/
	XADCInit(&XADCMonInst);


	u32 TempRawData;
	float TempData;

	/****************** Initialize CAN driver ***************************/
	CanInit(&Can, CAN_DEVICE_ID);

	/******* Initialize TmrCtr 0 ********/
	//TCInit(&TC0, TMRCTR_0_DEVICE_ID, TC0Handler, TIMER_COUNTER_0, RESET_VALUE0);

	/******* Initialize PWM @ TmrCtr 1 ********/
	Status = PWMInit(&TC0, TMRCTR_0_DEVICE_ID);
	if(Status != XST_SUCCESS){
		xil_printf("PWMInit fail... \n");
	}else{
		xil_printf("PWMInit ok! \n");
	}
	//PWMSetPeriod(&TC0, 1);
	//PWMSetDuty(&TC0, 1);


	xil_printf("starting while() \n");
	while(1){

    	gpio_input = XGpio_DiscreteRead(&Gpio, GPIO_IN_CH);
    	//xil_printf("Gpio input: %d \n", gpio_input);
    	if(gpio_input == 1 || gpio_input == 4){
			XGpio_DiscreteSet(&Gpio, GPIO_OUT_CH, 0b00000010);
			usleep(50000);
			XGpio_DiscreteSet(&Gpio, GPIO_OUT_CH, 0b00000100);
			usleep(50000);
			XGpio_DiscreteSet(&Gpio, GPIO_OUT_CH, 0b00010000);
			usleep(50000);
			XGpio_DiscreteSet(&Gpio, GPIO_OUT_CH, 0b00100000);
			usleep(50000);
			XGpio_DiscreteSet(&Gpio, GPIO_OUT_CH, 0b00010000);
			usleep(50000);
			XGpio_DiscreteSet(&Gpio, GPIO_OUT_CH, 0b00000100);
			usleep(50000);
			XGpio_DiscreteSet(&Gpio, GPIO_OUT_CH, 0b00000010);
			usleep(50000);
			XGpio_DiscreteSet(&Gpio, GPIO_OUT_CH, 0b00000000);
			usleep(50000);
    	}else{
    		TempRawData = XAdcPs_GetAdcData(&XADCMonInst, XADCPS_CH_TEMP);
			TempData = XAdcPs_RawToTemperature(TempRawData);
			printf("The Current Temperature is %0d.%03d Centigrades.\n",
					(int)(TempData), XAdcFractionToInt(TempData));
    	}

    	for(int i = 0; i < 100; i++){

    	}
    	//xil_printf("%d: ", loopcount);

    	/*Status = SendFrame(CanInstPtr);
		xil_printf("Send - status: \t %d \n", Status);

		/*Status = RecvFrame(CanInstPtr);
		xil_printf("Recv - status: \t %d \n", Status);
		sleep(1);*/
    }


    return 0;
}

int PWMInit(XTmrCtr *TCPtr, u32 DeviceID){
	/*
	 *  To enable PWM on an AXI TC module:
	 *  - mode for both timer 0 and 1 must be Generate (MDT in TCSR = 0)
	 *  - PWMA0 bit in both TCSR's set to 1
	 *  - GenerateOut enabled (GENT in TSCR = 1)
	 *  - Assertion level of the GenOut signals must be active HIGH (done in vivado)
	 */
	int Status;
	u32 CurReg0;
	u32 CurReg1;
	/** Device Init and Self-test**/
	Status = XTmrCtr_Initialize(TCPtr, DeviceID);
	Status = XTmrCtr_SelfTest(TCPtr, DeviceID);

	CurReg0 = XTmrCtr_ReadReg(TCPtr->BaseAddress, TIMER_COUNTER_0, XTC_TCSR_OFFSET);
		xil_printf("TC0 TCSR0: %x \t ", CurReg0);
	CurReg1 = XTmrCtr_ReadReg(TCPtr->BaseAddress, TIMER_COUNTER_1, XTC_TCSR_OFFSET);
		xil_printf("TC0 TCSR1: %x \n", CurReg1);

	// Set PWM bit and GENT bit in both timer counter registers.
	XTmrCtr_WriteReg(TCPtr->BaseAddress, TIMER_COUNTER_0, XTC_TCSR_OFFSET,
				CurReg0 | XTC_CSR_ENABLE_PWM_MASK | XTC_CSR_EXT_GENERATE_MASK);
	XTmrCtr_WriteReg(TCPtr->BaseAddress, TIMER_COUNTER_1, XTC_TCSR_OFFSET,
				CurReg1 | XTC_CSR_ENABLE_PWM_MASK | XTC_CSR_EXT_GENERATE_MASK);

	CurReg0 = XTmrCtr_ReadReg(TCPtr->BaseAddress, TIMER_COUNTER_0, XTC_TCSR_OFFSET);
		xil_printf("TC0 TCSR0: %x \t", CurReg0);
	CurReg1 = XTmrCtr_ReadReg(TCPtr->BaseAddress, TIMER_COUNTER_1, XTC_TCSR_OFFSET);
		xil_printf("TC0 TCSR1: %x \n", CurReg1);

	PWMSetPeriod(TCPtr, 10);
	CurReg0 = XTmrCtr_ReadReg(TCPtr->BaseAddress, TIMER_COUNTER_0, XTC_TLR_OFFSET);
		xil_printf("TC0 TLR0: %x \t", CurReg0);
	PWMSetDuty(TCPtr, 1);
	CurReg1 = XTmrCtr_ReadReg(TCPtr->BaseAddress, TIMER_COUNTER_1, XTC_TLR_OFFSET);
		xil_printf("TC0 TLR1: %x \n", CurReg1);


	XTmrCtr_Start(TCPtr, TIMER_COUNTER_0);
	XTmrCtr_Start(TCPtr, TIMER_COUNTER_1);

	CurReg0 = XTmrCtr_ReadReg(TCPtr->BaseAddress, TIMER_COUNTER_0, XTC_TCSR_OFFSET);
		xil_printf("TC0 TCSR0: %x \t", CurReg0);
	CurReg1 = XTmrCtr_ReadReg(TCPtr->BaseAddress, TIMER_COUNTER_1, XTC_TCSR_OFFSET);
		xil_printf("TC0 TCSR1: %x \n", CurReg1);

	return Status;
}

void PWMSetDuty(XTmrCtr *TCPtr, float Duty){
	/*
		 * 	When counting UP (UDT = 0) (default):
		 * 	PWM_Period 		= (MAX_COUNT - TLR0 + 2) * AXI_CLK_PERIOD
		 * 	PWM_HIGH_TIME 	= (MAX_COUNT - TLR1 + 2) * AXI_CLK_PERIOD
		 *
		 * 	When counting DOWN (UDT = 1):
		 * 	PWM_Period 		= (TLR0 + 2) * AXI_CLK_PERIOD
		 * 	PWM_HIGH_TIME 	= (TLR1 + 2) * AXI_CLK_PERIOD
		 *
		 * 	MAX_COUNT = 0xFFFFFFFF (32 bit counters)
		 */

	u32 TLR1Filler;
	/*
	 * 	duty = (0xFFFFFFFF - TLR0 + 2)/PWM_AXI_CLK_HZ
	 * 	(duty*PWM_AXI_CLK_HZ) = 0xFFFFFFFFF - TLR0 + 2
	 * 	duty*AXI_CLK - 0xFFFFFFFF - 2 = - TLR0
	 * 	0xFFFFFFFF - duty*AXI_CLK + 2 = TLR0
	 */

	float midval = (1/Duty)/(1/PWM_AXI_CLK_HZ);

	TLR1Filler = 0xFFFFFFFF - (u32)midval + 2;

	XTmrCtr_WriteReg(TCPtr->BaseAddress, TIMER_COUNTER_1, XTC_TLR_OFFSET, TLR1Filler);
}

void PWMSetPeriod(XTmrCtr *TCPtr, float pwmFreq){
	u32 TLR0Filler;
	/*
		 * 	When counting UP (UDT = 0) (default):
		 * 	PWM_Period 		= (MAX_COUNT - TLR0 + 2) * AXI_CLK_PERIOD
		 * 	PWM_HIGH_TIME 	= (MAX_COUNT - TLR1 + 2) * AXI_CLK_PERIOD
		 *
		 * 	When counting DOWN (UDT = 1):
		 * 	PWM_Period 		= (TLR0 + 2) * AXI_CLK_PERIOD
		 * 	PWM_HIGH_TIME 	= (TLR1 + 2) * AXI_CLK_PERIOD
		 *
		 * 	MAX_COUNT = 0xFFFFFFFF (32 bit counters)
	 */

	/*
	 * PWM_Period = (MAX_CNT - TLR0 + 2) * AXI_CLK_PERIOD
	 * PWM_PERIOD/AXI_CLK_P = MAX_CNT - TLR0 +2
	 * PWM_P/AXI_P = mid
	 * MAX - mid - 2 = TLR0
	 *
	 */


	/* PWM Freq of 25KHz example:
	 * 1/25000 = (TLR0+2)/PWM_AXI_CLK_HZ
	 * PWM_AXI_CLK_HZ/(TLR0+2) = 25000
	 * PWM_AXI_CLK_HZ/25000-2 = TLR0 = 1998
	 *
	 * 1/25000 = (0xFFFFFFFF - TLR0 + 2)/AXI_CLK_HZ
	 * 0xFFFFFFFF - PWM_AXI_CLK_HZ/25000 + 2 = TLR0 =
	 */
	float midval = (1/pwmFreq)/(float)(1/PWM_AXI_CLK_HZ);
	xil_printf("mid: %.10f \n", midval);

	//TLR0Filler = (PWM_AXI_CLK_HZ/pwmFreq);
	TLR0Filler = 0xFFFFFFFF - (u32)midval + 2;
	XTmrCtr_WriteReg(TCPtr->BaseAddress, TIMER_COUNTER_0, XTC_TLR_OFFSET, TLR0Filler);
}

int TCInit(XTmrCtr *TCPtr, u32 DeviceID, XTmrCtr_Handler funcPtr, u8 TCNum, u32 reset_val){
	int Status;

	/** Device Init and Self-test**/
	Status = XTmrCtr_Initialize(TCPtr, DeviceID);
	Status = XTmrCtr_SelfTest(TCPtr, DeviceID);

	/******* Set up interrupt system **********/
	XScuGic_Config *IntcConfig;
	IntcConfig = XScuGic_LookupConfig(INTC_DEVICE_ID);
	Status = XScuGic_CfgInitialize(&InterruptController, IntcConfig, IntcConfig->CpuBaseAddress);

	XScuGic_SetPriorityTriggerType(&InterruptController, TMRCTR_INT_ID, 0xA0, 0x3);

	Status = XScuGic_Connect(&InterruptController, TMRCTR_INT_ID, (Xil_ExceptionHandler)XTmrCtr_InterruptHandler, &TC0);
	XScuGic_Enable(&InterruptController, TMRCTR_INT_ID);


	// Exception stuff....
	Xil_ExceptionInit();
	Xil_ExceptionRegisterHandler(XIL_EXCEPTION_ID_INT,
			(Xil_ExceptionHandler) INTC_HANDLER,
			&InterruptController);
	Xil_ExceptionEnable();

	XTmrCtr_SetHandler(TCPtr, funcPtr, TCPtr);
	XTmrCtr_SetOptions(TCPtr, TCNum, XTC_INT_MODE_OPTION | XTC_AUTO_RELOAD_OPTION);
	XTmrCtr_SetResetValue(TCPtr, TCNum, reset_val);

	XTmrCtr_Start(TCPtr, TCNum);
	return Status;
}

int XADCInit(XAdcPs* XADCInstPtr){
	int Status;
	//XAdcPs *XADCInstPtr = &XADCMonInst;
	XAdcPs_Config *Adc_ConfigPtr;

	// Initialize XADC
	Adc_ConfigPtr = XAdcPs_LookupConfig(XPAR_AXI_XADC_0_DEVICE_ID);
		if (Adc_ConfigPtr == NULL) {
			xil_printf("fail 1");
			return XST_FAILURE;
		}

	Status = XAdcPs_CfgInitialize(XADCInstPtr,Adc_ConfigPtr,Adc_ConfigPtr->BaseAddress);
		if(Status != XST_SUCCESS){
			xil_printf("ADC INIT FAILED\n\r");
			return XST_FAILURE;
		}

	// XADC self-test
	Status = XAdcPs_CfgInitialize(XADCInstPtr, Adc_ConfigPtr, Adc_ConfigPtr->BaseAddress);
		if(Status != XST_SUCCESS){
			xil_printf("ADC INIT FAILED\n\r");
			return XST_FAILURE;
		}

	/*** Sequencer mode setup ***/
	// stop sequencer
	XAdcPs_SetSequencerMode(XADCInstPtr, XADCPS_SEQ_MODE_SINGCHAN);
	// disable alarms
	XAdcPs_SetAlarmEnables(XADCInstPtr, 0x0);
	// configure sequencer to just sample internal on chip parameters
	XAdcPs_SetSequencerMode(XADCInstPtr, XADCPS_SEQ_MODE_SAFE);
	// configure the channel enables we want to monitor
	XAdcPs_SetSeqChEnables(XADCInstPtr, XADCPS_SEQ_CH_TEMP|XADCPS_SEQ_CH_VCCINT);
	return Status;
}

int CanInit(XCanPs *CanInstPtr, u16 DeviceID){
		int Status;

		XCanPs_Config *Can_ConfigPtr;

		Can_ConfigPtr = XCanPs_LookupConfig(DeviceID);
		Status = XCanPs_CfgInitialize(CanInstPtr, Can_ConfigPtr, Can_ConfigPtr->BaseAddr);
		Status = XCanPs_SelfTest(CanInstPtr);
			if (Status != XST_SUCCESS) {
				return XST_FAILURE;
			}

		XCanPs_EnterMode(CanInstPtr, XCANPS_MODE_CONFIG);
		while(XCanPs_GetMode(CanInstPtr) != XCANPS_MODE_CONFIG);

		XCanPs_SetBaudRatePrescaler(CanInstPtr, TEST_BRPR_BAUD_PRESCALAR);
		XCanPs_SetBitTiming(CanInstPtr, TEST_BTR_SYNCJUMPWIDTH,
										TEST_BTR_SECOND_TIMESEGMENT,
										TEST_BTR_FIRST_TIMESEGMENT);
		// bit_rate = 100MHz / (1*(3+3+7))
		// freqBIT_RATE = freqCAN_REF_CLK / ((can.BRPR[BRP] + 1) * (3 + can.BTR[TS1] + can.BTR[TS2]))
		//
		XCanPs_EnterMode(CanInstPtr, XCANPS_MODE_NORMAL);
		while(XCanPs_GetMode(CanInstPtr) != XCANPS_MODE_NORMAL);
		return Status;
}

static int SendFrame(XCanPs *InstancePtr){
	u8 *FramePtr;
	int Status;

	TxFrame[0] = (u32)XCanPs_CreateIdValue((u32)TEST_MESSAGE_ID, 0, 0, 0, 0);
	TxFrame[1] = (u32)XCanPs_CreateDlcValue((u32)FRAME_DATA_LENGTH);

	FramePtr = (u8 *)(&TxFrame[2]);
	for(int i= 0; i < FRAME_DATA_LENGTH; i++){
		*FramePtr++ = (u8)i;
	}

	// wait for the Tx FIFO to empty before sending message
	xil_printf("CAN TX buffer... ");
	while(XCanPs_IsTxFifoFull(InstancePtr) == TRUE);
	xil_printf("Empty! ");
	Status = XCanPs_Send(InstancePtr, TxFrame);

	return Status;
}

static int RecvFrame(XCanPs *InstancePtr){
	u8 *FramePtr;
	int Status;

	// wait until a frame is received
	while(XCanPs_IsRxEmpty(InstancePtr) == TRUE);

	Status = XCanPs_Recv(InstancePtr, RxFrame);
	if(Status == XST_SUCCESS){
		if(RxFrame[0] != (u32)XCanPs_CreateIdValue((u32)TEST_MESSAGE_ID, 0, 0, 0, 0))
			return XST_LOOPBACK_ERROR;
		if ((RxFrame[1] & ~XCANPS_DLCR_TIMESTAMP_MASK) != TxFrame[1])
			return XST_LOOPBACK_ERROR;

		FramePtr = (u8 *)(&RxFrame[2]);
		for(int i = 0; i< FRAME_DATA_LENGTH; i++){
			if(*FramePtr++ != (u8)i){
				xil_printf("frame data %d faulty. \n", i);
				return XST_LOOPBACK_ERROR;
			}
		}
	}
	return Status;
}

void TC0Handler(){
	static int on;
	xil_printf("Interrupt from TC0! \n");
	if(on > 0){
		XGpio_DiscreteSet(&Gpio, GPIO_OUT_CH, 0b00000000);
		on = 0;
	}else{
		XGpio_DiscreteSet(&Gpio, GPIO_OUT_CH, 1 << 2);
		on = 1;
	}
}

